/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.core;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import com.unitesk.requality.core.attribute.AppliedEnumValue;
import com.unitesk.requality.core.attribute.Attribute;
import com.unitesk.requality.core.attribute.Attribute.ParameterAvailability;
import com.unitesk.requality.core.attribute.AttributeGenerator;
import com.unitesk.requality.core.attribute.AttributeType;
import com.unitesk.requality.core.attribute.EnumDefinitionValue;
import com.unitesk.requality.core.attribute.EnumValue;
import com.unitesk.requality.core.attribute.ListValue;
import com.unitesk.requality.core.nodeiterators.AttributeValueIteratorType;

public class NodeMapper {
    private NodeMapper() {
    }

    public static UUID getUUId(String nodeString) throws JSONException {
        if (nodeString == null) {
            return null;
        }
        JSONObject obj = new JSONObject(nodeString);
        return UUID.fromString(obj.getString("uuid"));
    }

    public static Collection<Attribute> getAttributes(UUID uuid, String nodeString) throws JSONException {
        if (nodeString == null) {
            return null;
        }
        Collection<Attribute> result = new HashSet<Attribute>();
        JSONObject obj = new JSONObject(nodeString);
        try {
            JSONObject attrObj = obj.getJSONObject("attributes");
            loadNewFormat(uuid, result, attrObj);
        } catch (Exception e) {
        	try {
        		JSONObject attrObj = obj.getJSONObject("attributes");
        		loadSemiOldFormat(uuid, result, attrObj);
        	} catch (Exception ex) {
        		try {
        			JSONArray attrObj = obj.getJSONArray("attributes");
        			loadOldFormat(uuid, result, attrObj);
        		} catch (Exception ex2) {
        			
        		}
        	}
        }
        return result;
    }
    private static void loadSemiOldFormat(UUID uuid, Collection<Attribute> result, JSONObject attr)
            throws JSONException {
        for (String key : JSONObject.getNames(attr)) {
            JSONObject a = attr.getJSONObject(key);
        	UUID origin = uuid;
            AttributeType type = AttributeType.valueOf(a.getString("type"));
            Object value = null;
            AttributeType ltype = null;
            if (type == AttributeType.BOOL) {
                String v = a.getString("value");
                if ("true".equalsIgnoreCase(v) && !"false".equalsIgnoreCase(v)) {
                    value = Boolean.valueOf(v);
                } else {
                    value = v;
                }
            } else if (type == AttributeType.LIST) {
                value = oldListFromJSON(a);
            } else if (type == AttributeType.ENUM_DEFINITION){
            	value = oldMapFromJSON(a);
            } else if (type == AttributeType.REFERENCE) {
            	value = a.getString("value");
            } else if (type == AttributeType.ENUM){
            	value = new AppliedEnumValue( a.getString("last_value"),null,
            						AttributeType.valueOf(a.getString("elements_type")),
            								a.getString("enum_key"));
            } else {
                // TODO: all attribute types support
                try {
                    value = type.fromString(a.getString("value"));
                } catch (Exception e) {
                    value = a.getString("value");
                }
            }
            
            try {
            	String or = a.getString("origin");
            	origin = UUID.fromString(or);
            } catch (Exception e) {
            }
            ParameterAvailability pa = ParameterAvailability.LOCAL;
            try{
            	String avail = a.getString("availability");
            	pa = ParameterAvailability.valueOf(avail);
            	
            }catch(Exception e){
            }
        	
            Attribute rattr;
            if(a.has("generator")){
            	JSONObject obj = a.getJSONObject("generator");
            	Map<String,Object> setts = new HashMap<String, Object>();
            	JSONObject set = obj.getJSONObject("settings");
            	JSONArray names = set.names();
            	if(names!=null)
            		for(int i = 0;i<names.length();i++){
            			try{
            				JSONObject jo = set.getJSONObject(names.getString(i));
            				setts.put(names.getString(i), listFromJSON(jo));
            			}
            			catch(JSONException e){
            				setts.put(names.getString(i), set.get(names.getString(i)));	
            			}
            		}	
            	AttributeType at = AttributeType.STRING;
            	if(obj.has("atribute_type"))
            		at = AttributeType.values()[Integer.parseInt(obj.getString("atribute_type"))];
                value = valueFromJSON(at, a);
                rattr = new Attribute(origin, type, key, value);
            	AttributeGenerator ag = new AttributeGenerator(rattr, UUID.fromString(obj.getString("uuid")),setts);
            	ag.setGeneratorType(AttributeValueIteratorType.fromId(obj.getString("generator_type")));
            	rattr = new Attribute(ag.getMasterUUId(),AttributeType.GENERATED,rattr.getKey(),rattr.getValue(),rattr.getRawValue(),ag);
            	//genObj.put("atribute_type", ag.getAttributeType().ordinal());
            	//genObj.put("atribute_list_type", ag.getAttributeListType().ordinal());
            }else 
            	 rattr = new Attribute(origin, type, key, value);
        	rattr.setAvailability(pa);
            result.add(rattr);
        }
    }
    private static Object oldListFromJSON(JSONObject a) throws JSONException {
    	ArrayList<Object> attrLst = new ArrayList<Object>();
        JSONArray vals = a.getJSONArray("value");
        AttributeType ltype = AttributeType.STRING;
        if(a.has("elements_type"))
        	ltype = AttributeType.valueOf(a.getString("elements_type"));
        if(ltype != AttributeType.LIST&&ltype != AttributeType.ENUM_DEFINITION)
        	for (int j = 0; j < vals.length(); j++) {
        		attrLst.add(ltype.fromString(vals.getString(j)));
        	}
        else
        {
        	if(ltype == AttributeType.LIST)
        		for (int j = 0; j < vals.length(); j++) {
        			attrLst.add(oldListFromJSON(vals.getJSONObject(j))); 
        		}
        	if(ltype == AttributeType.ENUM_DEFINITION)
        		for (int j = 0; j < vals.length(); j++) {
        			attrLst.add(oldMapFromJSON(vals.getJSONObject(j))); 
        		}
        }
        return new ListValue(ltype, attrLst);
	}

    private static Object oldMapFromJSON(JSONObject a) throws JSONException {
    	List<EnumValue> evals=new ArrayList<EnumValue>();
        JSONArray vals = a.getJSONArray("value");
        JSONArray valsComments = null;
        if(a.has("comments"))
        	valsComments = a.getJSONArray("comments");
        
        AttributeType ltype = AttributeType.STRING;
        if(a.has("elements_type"))
        	ltype = AttributeType.valueOf(a.getString("elements_type"));
        if(ltype != AttributeType.LIST&&ltype != AttributeType.ENUM_DEFINITION)
        	for (int j = 0; j < vals.length(); j++) {
        		evals.add(new EnumValue(vals.getString(j),valsComments==null?null:valsComments.getString(j).equals("")?null:valsComments.getString(j)));
        	}
        else
        {
        	if(ltype == AttributeType.LIST)
        		for (int j = 0; j < vals.length(); j++) {
        			evals.add(new EnumValue(oldListFromJSON(vals.getJSONObject(j)),valsComments==null?null:valsComments.getString(j).equals("")?null:valsComments.getString(j))); 
        		}

        	if(ltype == AttributeType.ENUM_DEFINITION)
        		for (int j = 0; j < vals.length(); j++) {
        			evals.add(new EnumValue(oldMapFromJSON(vals.getJSONObject(j)),valsComments==null?null:valsComments.getString(j).equals("")?null:valsComments.getString(j)));
        		}
        }

        if(a.has("comments")){
        	JSONArray comms = a.getJSONArray("value");
        	
        }
        return new EnumDefinitionValue(ltype, evals);
	}

    private static void loadOldFormat(UUID uuid, Collection<Attribute> result, JSONArray attr)
            throws JSONException {
        for (int i = 0; i < attr.length(); i++) {
            JSONObject a = attr.getJSONObject(i);
            AttributeType type = AttributeType.valueOf(a.getString("type"));
            String key = a.getString("key");
            Object value = null;
            if (type == AttributeType.LIST) {
                ArrayList<Object> attrLst = new ArrayList<Object>();
                JSONArray vals = a.getJSONArray("value");
                for (int j = 0; j < vals.length(); j++) {
                    attrLst.add(vals.getString(j));
                }
                value = new ListValue(AttributeType.STRING, attrLst);
            } else {
                // TODO: all attribute types support
                value = type.fromString(a.getString("value"));
            }
            result.add(new Attribute(uuid, type, key, value));
        }
    }

    private static void loadNewFormat(UUID uuid, Collection<Attribute> result, JSONObject attr)
            throws JSONException {
        for (String key : JSONObject.getNames(attr)) {
            JSONObject a = attr.getJSONObject(key);
        	UUID origin = uuid;
            AttributeType type = AttributeType.valueOf(a.getString("type"));
            Object value = null;
            AttributeType ltype = null;
            value = valueFromJSON(type, a);
            
            try {
            	String or = a.getString("origin");
            	origin = UUID.fromString(or);
            } catch (Exception e) {
            }
            ParameterAvailability pa = ParameterAvailability.LOCAL;
            try{
            	String avail = a.getString("availability");
            	pa = ParameterAvailability.valueOf(avail);
            	
            }catch(Exception e){
            }
            Attribute rattr;
            if(a.has("generator")){
            	JSONObject obj = a.getJSONObject("generator");
            	Map<String,Object> setts = new HashMap<String, Object>();
            	JSONObject set = obj.getJSONObject("settings");
            	JSONArray names = set.names();
            	if(names!=null)
            		for(int i = 0;i<names.length();i++){
            			try{
            				JSONObject jo = set.getJSONObject(names.getString(i));
            				setts.put(names.getString(i), listFromJSON(jo));
            			}
            			catch(JSONException e){
            				setts.put(names.getString(i), set.get(names.getString(i)));	
            			}
            		}
            	AttributeType at = AttributeType.STRING;
            	if(obj.has("attribute_type"))
            		at = AttributeType.values()[Integer.parseInt(obj.getString("attribute_type"))];
                value = valueFromJSON(at, a);
                rattr = new Attribute(origin, type, key, value);
            	AttributeGenerator ag = new AttributeGenerator(rattr, UUID.fromString(obj.getString("uuid")),setts);
            	ag.setGeneratorType(AttributeValueIteratorType.fromId(obj.getString("generator_type")));
            	rattr = new Attribute(ag.getMasterUUId(),AttributeType.GENERATED,rattr.getKey(),rattr.getValue(),rattr.getRawValue(),ag);
            	//genObj.put("atribute_type", ag.getAttributeType().ordinal());
            	//genObj.put("atribute_list_type", ag.getAttributeListType().ordinal());
            }else
            	rattr = new Attribute(origin, type, key, value);
        	rattr.setAvailability(pa);
            result.add(rattr);
        }
    }

    private static Object valueFromJSON(AttributeType type, JSONObject a) throws JSONException {
    	Object value;
    	if (type == AttributeType.BOOL) {
            String v = a.getString("value");
            if ("true".equalsIgnoreCase(v) && !"false".equalsIgnoreCase(v)) {
                value = Boolean.valueOf(v);
            } else {
                value = v;
            }
        } else if (type == AttributeType.LIST) {
            value = listFromJSON(a);
        } else if (type == AttributeType.ENUM_DEFINITION){
        	value = enumDefinitionFromJSON(a);
        } else if (type == AttributeType.REFERENCE) {
        	value = a.getString("value");
        } else if (type == AttributeType.ENUM){
        	value = appliedEnumFromJSON(a);
        } else {
            // TODO: all attribute types support
            try {
                value = type.fromString(a.getString("value"));
            } catch (Exception e) {
                value = a.getString("value");
            }
        }
    	return value;
	}

	private static AppliedEnumValue appliedEnumFromJSON(JSONObject a) throws JSONException {
		AttributeType at = AttributeType.valueOf(a.getString("elements_type"));
		EnumValue ev = enumFromJSON(at, a);
		return new AppliedEnumValue(ev.getValue(), ev.getComment(),at, a.getString("enum_key"));
	}

	private static Object listFromJSON(JSONObject a) throws JSONException {
		AttributeType ltype = AttributeType.STRING;
        if(a.has("elements_type"))
        	ltype = AttributeType.valueOf(a.getString("elements_type"));
    	ArrayList<Object> attrLst = new ArrayList<Object>();
        JSONArray vals = a.getJSONArray("value");
        for (int j = 0; j < vals.length(); j++){
        	Object o = vals.get(j);
        	if(o instanceof JSONObject)
        		attrLst.add(valueFromJSON(ltype, vals.getJSONObject(j)));
        	else
        		attrLst.add(o);//backward comp
        }
        return new ListValue(ltype,attrLst);
	}
    private static EnumValue enumFromJSON(AttributeType at,JSONObject a) throws JSONException {
    	Object val = valueFromJSON(at, a);
    	String comm = null;
    	if(a.has("comment"))
    		comm = a.getString("comment");
		return new EnumValue(val, comm);
    }
        
    private static EnumDefinitionValue enumDefinitionFromJSON(JSONObject a) throws JSONException {
        JSONArray vals = a.getJSONArray("value");
        
        AttributeType ltype = AttributeType.STRING;
        if(a.has("elements_type"))
        	ltype = AttributeType.valueOf(a.getString("elements_type"));
        
        EnumDefinitionValue edv = new EnumDefinitionValue(ltype, new ArrayList<EnumValue>());

        for(int j = 0;j<vals.length();j++)
        	edv.addEnumValue(enumFromJSON(ltype, vals.getJSONObject(j)));
        
//        if(ltype != AttributeType.LIST&&ltype != AttributeType.ENUM_DEFINITION)
//        	for (int j = 0; j < vals.length(); j++) {
//        		attrLst.put(ltype.fromString(vals.getString(j)),valsComments==null?null:valsComments.getString(j).equals("")?null:valsComments.getString(j));
//        	}
//        else
//        {
//        	if(ltype == AttributeType.LIST)
//        		for (int j = 0; j < vals.length(); j++) {
//        			attrLst.put(listFromJSON(vals.getJSONObject(j)),valsComments==null?null:valsComments.getString(j).equals("")?null:valsComments.getString(j)); 
//        		}
//
//        	if(ltype == AttributeType.ENUM_DEFINITION)
//        		for (int j = 0; j < vals.length(); j++) {
//        			attrLst.put(enumDefinitionFromJSON(vals.getJSONObject(j)),valsComments==null?null:valsComments.getString(j).equals("")?null:valsComments.getString(j));
//        		}
//        }

        return edv;
	}

	public final static JSONObject toJSON(TreeNode node) throws JSONException {
    	return toJSON(node.getUUId(), node.getAttributes().values());
    }

    @SuppressWarnings("unchecked")
    public final static JSONObject toJSON(UUID uuid, Collection<Attribute> attr)
            throws JSONException {
        JSONObject result = null;
        result = new JSONObject();
        result.put("uuid", uuid);
        JSONObject attrs = new JSONObject();
        for (Attribute a : attr) {
            JSONObject attrObj = new JSONObject();
            attrObj.put("isGenerated", a.isGenerated());
            attrObj.put("key", a.getKey());
            attrObj.put("type", a.getRawType());
            Object origin = a.getOrigin();
            if(origin == null)
            	origin = "";
            attrObj.put("origin", origin);
            valueToJSON(attrObj, a.getRawType(), a.getResultType(),a.getRawValue(),a.getValue());

            attrObj.put("availability", a.getAvailability().name());
            AttributeGenerator ag = a.getGenerator();
            if(ag!=null){
            	JSONObject genObj = new JSONObject();
            	genObj.put("uuid", ag.getUUID());
            	genObj.put("attribute_type", ag.getAttributeType().ordinal());
            	if(ag.getAttributeListType()!=null)
            		genObj.put("atribute_list_type", ag.getAttributeListType().ordinal());
            	genObj.put("generator_type", ag.getGeneratorType().getId());
            	JSONObject genAttrs = new JSONObject();
            	Map<String,Object>setts = ag.getSettings();
            	for(String s:setts.keySet())
            		if(setts.get(s) instanceof ListValue){
            			JSONObject val = new JSONObject();
            			valueToJSON(val, AttributeType.LIST,AttributeType.LIST, (ListValue)setts.get(s),null);
            			genAttrs.put(s, val);
            		}
            		else
            			genAttrs.put(s, setts.get(s));
            	genObj.put("settings", genAttrs);
            	attrObj.put("generator", genObj);
            }
            attrs.put(a.getKey(), attrObj);
        }
        result.put("attributes", attrs);
        return result;
    }
	private static void valueToJSON(JSONObject attrObj,
			AttributeType rawType, AttributeType resType, Object rawValue,Object newValue) throws JSONException {
		 if (rawType.equals(AttributeType.ENUM)){
			 AppliedEnumValue aev = (AppliedEnumValue)rawValue;
         	 attrObj.put("elements_type", aev.getType());
         	 attrObj.put("enum_key", aev.getBaseKey());
         	 valueToJSON(attrObj, aev.getType(),aev.getType(), aev.getValue(),null);
 	        if(newValue!=null)
 	        	attrObj.put("value",newValue);
         }else
         if (resType == AttributeType.LIST) {
         	JSONArray lst = new JSONArray();
         	ListValue lv = (ListValue)rawValue;
	        //Object[]val = (Object[])rawValue;
	        AttributeType at = lv.getType();
	        attrObj.put("elements_type", at);
          	for(Object o:lv){
          		JSONObject jso = new JSONObject();
          		valueToJSON(jso, at, at, o,null);
          		lst.put(jso);
          	}
	        attrObj.put("value", lst);
	        //TODO new value
         }
 	     else if (rawType == AttributeType.ENUM_DEFINITION) {
 	        JSONArray lst = new JSONArray();
 	        EnumDefinitionValue edv = (EnumDefinitionValue)rawValue;
	        attrObj.put("elements_type", edv.getType());
	        for(EnumValue ev:edv.getValues())
	          	lst.put(enumValueToJSON(edv.getType(), ev));
	        attrObj.put("value", lst);
	        //TODO new value
 	 	 } else {
 	 	  	attrObj.put("value", rawValue);
 	 	  	if(newValue!=null)
 	        	attrObj.put("newvalue", newValue);	
 	 	 }
	}

	private static JSONObject enumValueToJSON(AttributeType at,EnumValue ev) throws JSONException {
		JSONObject jo = new JSONObject();
		if(ev.getComment()!=null)
			jo.put("comment", ev.getComment());
    	valueToJSON(jo, at,at, ev.getValue(),null);
		return jo;
	}
}
