package com.unitesk.requality.eclipse.core;

import java.util.ArrayList;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.WorkspaceJob;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.core.checkers.CheckerContainer;
import com.unitesk.requality.core.checkers.CheckerProblem;

public class EclipseCheckerContainer extends CheckerContainer {
    private static final String REQUALITY_MARKER_TYPE = "com.unitesk.requality.marker";

    @Override
    public void check(final TreeNode node, boolean batch) {
    	if(batch){
    		EclipseCheckerContainer.this.doCheck(node);
    	}else{
    		WorkspaceJob checkNode = new WorkspaceJob("Check Node " + node.getId()) {
    			@Override
            	public IStatus runInWorkspace(IProgressMonitor monitor) {
                	try {
                		EclipseCheckerContainer.this.doCheck(node);
                	} catch (Exception e) {
                    	e.printStackTrace();
                    	return Status.CANCEL_STATUS;
                	}
                	return Status.OK_STATUS;
            	}
    		};
    		//checkNode.setRule(ResourcesPlugin.getWorkspace().getRuleFactory().buildRule());
    		checkNode.setPriority(WorkspaceJob.LONG);
    		checkNode.schedule();
    	}
    }

    @Override
    protected void applyProblemsInfo(final ArrayList<CheckerProblem> result) {
        IResource res = TreesTracker.getResourceByNode((TreeNode) getNode());
        if(res == null || !res.exists()) {
            // FIXME: why we can be here?
            return;
        }
        try {
            res.deleteMarkers(REQUALITY_MARKER_TYPE, true, IResource.DEPTH_ONE);
        } catch (CoreException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        for (CheckerProblem problem : result) {
            TreeNode node = (TreeNode) problem.getTarget();
            IResource resource = TreesTracker.getResourceByNode(node);
            if (resource == null || !resource.exists())
                // FIXME: why we can be here?
                continue;
            try {
                IMarker marker = resource.createMarker(REQUALITY_MARKER_TYPE);
                marker.setAttribute(IMarker.MESSAGE, problem.getDescription());
                marker.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_ERROR);
                marker.setAttribute(IMarker.LOCATION, node.toString());
            } catch (CoreException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
    }
}
