package com.unitesk.requality.eclipse.handlers;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;

import com.unitesk.requality.core.IDeleteHandler;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.nodetypes.ErrorNode;

public class DefaultDeleteNode extends AbstractHandler implements IDeleteHandler {

    public void recUpdateProcessed(TreeNode node, List<UUID> toprocess){
        toprocess.add(node.getUUId());
        for(UUID uid :node.getChildrenUUIds()){
            TreeNode nde = node.getTreeDB().getNode(uid);
            recUpdateProcessed(nde, toprocess);
        }
    }

    @Override
    public boolean isSuitableFor(TreeNode tn) {
    	Map<String,IDeleteHandler> dhlist = Activator.getDefault().getDeleteHandlers();
    	Iterator<String> dhit = dhlist.keySet().iterator();
        while(dhit.hasNext()){
            String dhname = dhit.next();
            IDeleteHandler idh = dhlist.get(dhname); 
            if(!(idh instanceof DefaultDeleteNode) && idh.isSuitableFor(tn))
                return false;
        }
        return true;
    }

    @Override
    public void process(List<TreeNode> nodes, List<UUID> processed, boolean multi)
            throws ExecutionException {
        for(TreeNode node:nodes){
            if(processed.contains(node.getUUId()))
                continue;
            if (!node.getTreeDB().getTreeLogic().canBeDeleted(node))
                continue;
            List<UUID>toprocess = new ArrayList<UUID>();
            recUpdateProcessed(node, toprocess);
            node.delete();
            processed.addAll(toprocess);
        }
    }
	@Override
	public String[] getWarnings(List<TreeNode> nodes) {
		return null;
	}
	@Override
	public String titleOverride(List<TreeNode> nodes) {
		return null;
	}

	@Override
	public Object execute(ExecutionEvent event) throws ExecutionException {
		return null;
	}

}
