package com.unitesk.requality.eclipse.handlers;

import java.util.List;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.ITreeLogic.IdContainsWrongChars;
import com.unitesk.requality.core.ITreeLogic.IdEmpty;
import com.unitesk.requality.core.ITreeLogic.PastingIntoLeafException;
import com.unitesk.requality.core.ITreeLogic.SameIdExistsInChildren;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.tools.Index;
import com.unitesk.requality.tools.RequalityCLI;

public abstract class NewElementHandler extends AbstractHandler {
	public String getElementType(){
		String[]types = getElementTypes();
		if(types!=null && types.length==1)
			return types[0];
		return null;
	}
	public TreeDB tree;
	public abstract String[] getElementTypes();
	public String getShowedElementType(){
		return getShowedElementType(getElementType());
	}
	public String getShowedElementType(String tp){
		return tree.getNodeType(tp).getVisibleClassName();
	}
	protected String getNewId(TreeNode par, String type){
		return RequalityCLI.getNextId(par, type, null);
	}

	public <type extends TreeNode> type execute(TreeNode par){
		return execute(par,TreeDB.EMPTY_TEMPLATE,getElementType());
	}
	public <type extends TreeNode> type execute(TreeNode par, String templ, String type){
		tree = par.getTreeDB();
        TreeDB db = par.getTreeDB();
        TreeNode node = null;
        if( !db.getTreeLogic().canBeChildOf(type, par.getType()) &&
        		db.getTreeLogic().canBeChildOf(type, par.getParent(true).getType())
        		|| RequalityCNF.shiftPressed){
        	node = par;
        	par = par.getParent(true);
        }

		String id = getNewId(par,type);

		if(id == null)
			return null;
		  try {
	            if(!par.getTreeDB().getTreeLogic().canBeCreatedInExc(type, id, par)){
	                MessageDialog.openError(PlatformUI.getWorkbench()
	                        .getActiveWorkbenchWindow().getShell(), "Create Element",
	                        "Element of type '" + getShowedElementType() 
	                        + "' cannot be created in " + par.getUserFriendlyName());
	                return null;
	            }
	        } catch (PastingIntoLeafException e) {
	            MessageDialog.openError(PlatformUI.getWorkbench()
	                    .getActiveWorkbenchWindow().getShell(), "Create Element",
	                    "Can't add "+getShowedElementType()+" to "+par.getTypeLabel()+" with "
	                    + "Test Purpose child");
	            return null;
	        } catch (SameIdExistsInChildren e) {
	            MessageDialog.openError(PlatformUI.getWorkbench()
	                    .getActiveWorkbenchWindow().getShell(), "Create Element",
	                    "Element with same id exists in " + 
	                    		par.getUserFriendlyName());
	            return null;
	        } catch (IdEmpty e) {
	            MessageDialog.openError(PlatformUI.getWorkbench()
	                    .getActiveWorkbenchWindow().getShell(), "Create Element",
	                    "Element id is empty");
	            return null;
	        } catch (IdContainsWrongChars e) {
	            MessageDialog.openError(PlatformUI.getWorkbench()
	                    .getActiveWorkbenchWindow().getShell(), "Create Element",
	                    "Element id contains wrong character - " 
	                    + e.getWrongCharacter());
	            return null;
	        }
	        if(db.startTransaction("Create "+getShowedElementType(type),true)){
	        	Index ind = Index.getNextIndex(type, par, node, false);
	        	TreeNode nn = par.createChild(id, type, templ);
	        	nn.setIndex(ind);
	        	setElementSpecificAttributes(nn);
	        	nn.saveAttributes();
	        	db.commit();
	        	if(Activator.getDefault()!=null)
	        		Activator.getDefault().setActiveNode(nn, type);
	        	return (type)nn;
	        }
	        return null;
	}
	protected abstract void setElementSpecificAttributes(TreeNode nn);
	@Override
	public Object execute(ExecutionEvent event) throws ExecutionException {
		List<TreeNode> nodes = RequalityCNF.getSelectedNodes();
		if (nodes == null || nodes.size() > 1)
		     return null;
		return execute(nodes.get(0), null, getElementType());
	}

	@Override
	public boolean isEnabled() {
	    List<TreeNode> nodelist = RequalityCNF.getSelectedNodes();
	    if (nodelist == null || nodelist.size() > 1)
	        return false;
	    if(nodelist.get(0).isVirtual())
	    	return false;
	    tree = nodelist.get(0).getTreeDB();
	    if (!checkElementSpecificConditions(nodelist.get(0)))
	    	return false;
	    for(String s:getElementTypes())
	    	if (nodelist.get(0).getTreeDB().getTreeLogic().canBeCreatedIn(s, nodelist.get(0)))
	    		return true;
	    return false;
	}
	protected boolean checkElementSpecificConditions(TreeNode treeNode) {
		return true;
	}
}
