/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.handlers.report;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.nio.charset.Charset;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IHandler;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchWindow;

import com.unitesk.requality.core.NodeDesc;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.core.attribute.Attribute;
import com.unitesk.requality.core.attribute.AttributeType;
import com.unitesk.requality.core.instancer.TreeInstancer;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.NodeTypeDescriptor;
import com.unitesk.requality.eclipse.RequalityPlugin;
import com.unitesk.requality.eclipse.core.TreesTracker;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.nodetypes.ReportData;
import com.unitesk.requality.nodetypes.ReportSettings;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.nodetypes.VirtualNode;
import com.unitesk.requality.report.ReportGenerator;
import com.unitesk.requality.report.Template;
import com.unitesk.requality.report.TemplateFile;
// TODO report undo
public class GenerateReportHandler extends AbstractHandler implements IHandler {

    public static IContainer getReportFolder(TreeNode node) {
        IFile nodeFile = (IFile) TreesTracker.getResourceByNode(node);
        IContainer res = nodeFile.getParent().getFolder(
                Path.fromOSString(node.getId() + "_resources"));
        return res;
    }

    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        List<TreeNode> nodes = RequalityCNF.getSelectedNodes();
        Map par = event.getParameters();
        if(nodes == null && par != null)
            return null;
        if(nodes == null){
        	nodes = new LinkedList<TreeNode>();
        	TreeNode t = (TreeNode) par.get("target");
        	if(t!=null)
        		nodes.add(t);
        }

       // nodes.get(0).getTreeDB().startTransaction("Generate Reports");
        for(TreeNode node:nodes){
            ReportSettings report = null;
            if (node instanceof ReportSettings) {
                report = (ReportSettings) node;
                generateReport(report); 
            }
        }
        //nodes.get(0).getTreeDB().commit();
        return null;
    }
    private static Shell getShell(){
    	if(Activator.getDefault() == null)
    		return null;
    	IWorkbench w = Activator.getDefault().getWorkbench();
    	if(w==null)
    		return null;
    	IWorkbenchWindow win = w.getActiveWorkbenchWindow();
    	if(win == null)
    		return null;
    	return win.getShell();
    }
    private static void openInformation(String title, String message){
    	Shell s = getShell();
    	if(s == null)
    		return;
    	MessageDialog.openInformation(
                s,
                title,
                message);
    }
    private static void openError(String title, String message){
    	Shell s = getShell();
    	if(s == null)
    		return;
    	MessageDialog.openError(
                s,
                title,
                message);
    }
    public static String generateReport(ReportSettings reportS) {
        TreeDB db = reportS.getTreeDB();
        NodeDesc nd = reportS.toNodeDesc(true);
        String date = new Date(
                System.currentTimeMillis()).toString();
        nd.setAttribute(new Attribute(nd.getUUId(),AttributeType.STRING,"_type",ReportData.TYPE_NAME));
        String href = null;
        TreeInstancer oldti = null,ti=null;
        TreeDB instTree=null, olddb = null;
        try{
	        Template tpl = null;
	        tpl = getTemplate(reportS.getTemplateId());
	        if (tpl == null) {
	            if(reportS.getTemplateId() != null){
	            	openError("Report generation",
	                    "Report template '" + reportS.getTemplateId() + "' for this report is not avalible.");
	                return null;
	            }
	            tpl = Activator.getDefault().getReportTemplates().get(Template.DEFAULT_ID);
	        }
	        //nd.setId((reportS.getId()+" ["+tpl.getName()+"] ("+date+")").replace(' ', '_').replace(':', '-'));
	        SimpleDateFormat dt = new SimpleDateFormat("yyyy-MM-dd_hh-mm-ss"); 
	        nd.setId((reportS.getId()+"("+dt.format(new Date(System.currentTimeMillis())))+")");
	        ReportData report = (ReportData) nd.createNode(db, reportS.getParentUUId(true), true,false);
	        Map<String,Object> objMap=new HashMap<String, Object>();
	        for(String s:report.getAttributes().keySet())
	        	objMap.put(s, report.getAttribute(s).getValue().toString());
	        oldti = VirtualNode.getVirtualStorage(db);
	        oldti.setReadOnly(true);
	        ti = VirtualNode.getVirtualStorage(db,true,objMap,oldti);
	        ti.setReadOnly(true);
	        instTree = TreeDB.load(ti);
	        instTree.setVirtualState(true);
	        //TreeDB instTree = TreeDB.load(ti);
	        olddb = VirtualNode.getVirtualTreeDB(db);
	        VirtualNode.markInstanceDB(instTree, db,ti);
	        Map<String, NodeTypeDescriptor> nts = RequalityPlugin
	                .getAdditionalTypes();
	        for (String name : nts.keySet()) {
	            instTree.addNodeType(name, nts.get(name).getClazz());
	        }
	        try{
	        	instTree.getNode(report
	                .getStartRequirementQId()); 
	        	instTree.updateRequirementsNames();
	        }catch (RuntimeException re){
	        	openInformation("Report generator error", "Target Requirement does not exist.");
	        	return null;
	        }
	        
            String savePath = report.getResourcesPath();
            //TODO look
            /*
			instTree.setReadOnlyState(false);
            hideInstancers(instTree.getNode(node
                    .getStartRequirementQId()));
            instTree.setReadOnlyState(true);
			*/
            Map<TemplateFile, Writer> bufs = new HashMap<TemplateFile, Writer>();
            Map<TemplateFile, ByteArrayOutputStream> baos = new HashMap<TemplateFile, ByteArrayOutputStream>();
            for(TemplateFile tf:tpl.getTemplates()){
            	baos.put(tf, new java.io.ByteArrayOutputStream());
            	bufs.put(tf, new OutputStreamWriter(baos.get(tf),Charset.forName("UTF-8")));
            }
            Requirement r = (Requirement) instTree.getNode(report
                    .getStartRequirementQId());
            if(r == null)
            	for(UUID uid:instTree.getRootNode().getRawChildrenUUIDs())
            	{
            		TreeNode n = instTree.getNode(uid);
            		if(Requirement.TYPE_NAME.equals(n.getType()))
            			r = (Requirement)n;
            	}
            ReportGenerator.generateReport(r
                    , tpl, bufs, 
                    savePath, new HashMap<String, Object>(report.getData()));
            for(TemplateFile tf:tpl.getTemplates())
            	report.setResourceContent(tf.getOutputFileName(), new ByteArrayInputStream(baos.get(tf).toByteArray()));
            // TODO: index file selection
            href = savePath + File.separator
                    + tpl.getTemplates().get(0).getOutputFileName();

            IProject project = TreesTracker.getTracker().getProject(db);
            IPath path = Path.fromOSString(savePath).makeRelativeTo(
                    project.getLocation());

            IFolder folder = project.getFolder(path);
            folder.refreshLocal(IResource.DEPTH_INFINITE, null);
	        if(href == null)
	        	return null;
	        // TODO report undo
	//        node.getTreeDB().startTransaction("Generate Report");
	        if(report.getAttributeValue(ReportSettings.ATTR_TEMPLATE) == null)
	        	report.putAttribute(new Attribute(report, AttributeType.STRING, ReportSettings.ATTR_TEMPLATE,tpl.getId()));
	        report.putAttribute(new Attribute(report, AttributeType.STRING, ReportSettings.ATTR_DATE, date));
	        report.putAttribute(new Attribute(report, AttributeType.STRING, ReportSettings.ATTR_LAST_GENERATED,
	                href.substring(href.lastIndexOf(java.io.File.separatorChar))));
	        report.saveAttributes();
	        String s = ti.getErrors();
	        if(!s.equals(""))
	        	openInformation("Report Generator",
	                "Warnings: "+s);
	//        node.getTreeDB().commit();
	        OpenReportHandler.openReport(report);
        } catch (Exception e) {
            e.printStackTrace();
        }
        finally{
        	if(oldti!=null){
    	        VirtualNode.clearInstanceInfo(instTree, db, ti);
    	        VirtualNode.markInstanceDB(olddb, db, oldti);
        		oldti.setReadOnly(false);
        	}
        }
        return href;
    }

    public static Template getTemplate(ReportSettings report){
    	String templateId = report.getTemplateId();
        Template tpl = null;
        if(templateId == null){
            tpl = Activator.getDefault().getReportTemplates()
                    .get(Template.DEFAULT_ID);
            report.putAttribute(new Attribute(report,AttributeType.STRING, ReportSettings.ATTR_TEMPLATE,tpl.getId()));
            report.saveAttributes();
        }else{
            tpl = Activator.getDefault().getReportTemplates()
                .get(templateId);
        }
        return tpl;
    }
    public static Template getTemplate(String tid) {
        return Activator.getDefault().getReportTemplates().get(tid);
    }
//remove old image from report before new generation
	public static void clearOldImages(TreeNode nde) {
		if(!nde.getType().equals(ReportSettings.TYPE_NAME))
			return;
		ReportSettings node = (ReportSettings)nde;
        Template tpl = null;
        try {
            tpl = getTemplate(node.getTemplateId());
            if (tpl == null) {
                if(node.getTemplateId() != null){
                    return;
                }
                tpl = Activator.getDefault().getReportTemplates().values()
                        .iterator().next();
            }
            String savePath = node.getSaveDir();
            for (TemplateFile t : tpl.getTemplates()) {
                File outFile = new File(savePath + File.separator
                        + t.getOutputFileName());
                ReportGenerator.clearOldImages(outFile, savePath + File.separator);
            }
             // TODO: index file selection
        } catch (Exception e) {
            e.printStackTrace();
        }
	}
	@Override
	public boolean isEnabled() {
        List<TreeNode> nodes = RequalityCNF.getSelectedNodes();
        if(nodes == null)
            return false;
        for(TreeNode node:nodes)
        	if(!node.getType().equals(ReportSettings.TYPE_NAME))
        		return false;
		return super.isEnabled();
	}
	
}
