/*
 * Copyright 2009-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.ui.dialogs;

import java.util.ArrayList;
import java.util.Collections;
import java.util.EventObject;
import java.util.Map;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.fieldassist.ControlDecoration;
import org.eclipse.jface.fieldassist.FieldDecoration;
import org.eclipse.jface.fieldassist.FieldDecorationRegistry;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ColumnViewerEditorActivationEvent;
import org.eclipse.jface.viewers.ColumnViewerEditorActivationStrategy;
import org.eclipse.jface.viewers.FocusCellOwnerDrawHighlighter;
import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerEditor;
import org.eclipse.jface.viewers.TableViewerFocusCellManager;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.DragSourceListener;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.DropTargetListener;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.attribute.Attribute;
import com.unitesk.requality.core.attribute.AttributeType;

/**
 * Attribute edit dialog
 */
public class EditAttrDialog extends Dialog {
	private boolean isZero(String val, boolean onedot){
		val = val.trim();
		if(val.length() > 0 && val.charAt(0)=='-')
			val = val.substring(1);
		int dc = 0;
		for(int i=0;i<val.length();i++)
			if(val.charAt(i)!='0'){
				if(val.charAt(i)=='.'){
					dc++;
					if(dc>1)
						return false;
				}else
					return false;
			}
		return true;
	}
	private String getValueError(String val){
		String err = null;
		if(targetType == AttributeType.BOOL || targetType == AttributeType.FLOAT || targetType == AttributeType.INT){
        	if(targetType == AttributeType.BOOL && !val.toLowerCase().equals("true") && !val.toLowerCase().equals("false"))
        		err = BOOL_ERROR;
        	if(targetType == AttributeType.FLOAT){
        		String oe = err;
        		err = FLOAT_ERROR;
        		try{
        			if((Math.abs(Float.parseFloat(val)))>0.0001 || isZero(val, true))
        				err = oe;
        		}catch(NumberFormatException er){
        			err = FLOAT_ERROR;
        		}
        	}
        	if(targetType == AttributeType.INT){
        		String oe = err;
        		err = INT_ERROR;
        		try{
        			if((Math.abs(Integer.parseInt(val)))>0 || !isZero(val, false))
        				err = oe;
        		}catch(NumberFormatException er){
            		err = INT_ERROR;
        		}
        	}
        }
		return err;
	}
    ModifyListener updateDecorator = new ModifyListener() {
        @Override
        public void modifyText(ModifyEvent e) {
            String err = EditAttrDialog.this.getNameError(name.getText().trim()),
            	   err2 = null;
            if(value != null)
            	EditAttrDialog.this.getValueError(((Text)value).getText());
            if(getButton(Dialog.OK)!=null) {
                if (err != null || err2 != null) {
                    getButton(Dialog.OK).setEnabled(false);
                } else {
                    getButton(Dialog.OK).setEnabled(true);
                }
            }
            if (err != null) {
                attrNameDecorator.setDescriptionText(err);
                attrNameDecorator.show();
                return;
            }
            attrNameDecorator.hide();
        }
    };
    ModifyListener valueUpdateDecorator = new ModifyListener() {
        @Override
        public void modifyText(ModifyEvent e) {
            String err2 = EditAttrDialog.this.getNameError(name.getText().trim()), 
            		err = EditAttrDialog.this.getValueError(((Text)value).getText());
            if(getButton(Dialog.OK)!=null) {
                if (err != null || err2 != null) {
                    getButton(Dialog.OK).setEnabled(false);
                } else {
                    getButton(Dialog.OK).setEnabled(true);
                }
            }
            if (err != null) {
                attrValueDecorator.setDescriptionText(err);
                attrValueDecorator.show();
                return;
            }
            attrValueDecorator.hide();
        }
    };
    static String TITLE = "Attribute Editor";
    static String LBL_NAME = "Input a name of the attribute:";
    static String LBL_VALUE = "Input a value of the attribute:";
    static String LBL_VALUE2 = "Attribute values:";

    static String MES_EXIST = "Attribute with the same name already exists";
    static String MES_EMPTY = "Name of the attribute can not be empty";
    static String MES_WRONG_FORMAT = "Name contains illegal characters or starts with _ or $";

    static String BOOL_ERROR = "Value of BOOL attribute must be 'true' or 'false'";
	static String FLOAT_ERROR = "Value of FLOAT attribute must be in numerical format with '.' delimeter";
	static String INT_ERROR = "Value of INT attribute must be in numerical format";
	
    static int SINGLE_WIDTH = 267;
    ControlDecoration attrNameDecorator, attrValueDecorator;
    private Text name = null;
    private Text value = null;
    private TableViewer table = null;
    private Button del = null;

    private Object[] initalVal = new Object[2];
    private Object[] curVal = new Object[2];
    private AttributeType targetType;

    private Map<String, Object> attrs;
    private boolean createMode = true;

    private ControlDecoration createDecorator(Text text) {
        ControlDecoration controlDecoration = new ControlDecoration(text,
                SWT.LEFT | SWT.TOP);
        FieldDecoration fieldDecoration = FieldDecorationRegistry.getDefault()
                .getFieldDecoration(FieldDecorationRegistry.DEC_ERROR);
        controlDecoration.setImage(fieldDecoration.getImage());
        return controlDecoration;
    }

    private final Listener keyLink = new Listener() {
        public void handleEvent(Event event) {
            if ((event.type == SWT.KeyDown) && (event.keyCode == SWT.CR)) {
                if (name.isFocusControl() == true) {
                    value.setFocus();
                } else {
                    if (value.isFocusControl() == true)
                        okPressed();
                }
            }
        }
    };

    public EditAttrDialog(Shell shell, AttributeType at, String name, Object value,
            Map<String, Object> attrs) {
        this(shell, at, name, value, attrs, true);

    }
    public EditAttrDialog(Shell shell, AttributeType at, String name, Object value,
            Map<String, Object> attrs, boolean createMode) {
        super(shell);

        curVal[0] = name;
        curVal[1] = value;
        initalVal[0] = name;
        initalVal[1] = value;
        
        targetType = at;

        this.attrs = Collections.unmodifiableMap(attrs);
        this.createMode = createMode;
    }

    public String getKey() {
        return (String)curVal[0];
    }

    public Object getValue() {
        return curVal[1];
    }

	private static long MAXATTRID = 0; 
    private class AttrItem{
    	private String value;
    	private long ind = MAXATTRID++;
    	public AttrItem(String val)
    	{
    		value = val;
    		if(val.equals(""+(char)10))
    			value = "";
    	}
    	public String dataString(){
    		if(value.equals(""))
    			return ""+(char)10;
    		return value;
    	}
    	@Override 
    	public String toString() {
    		return value;
    	}
    	public long Id(){
    		return ind;
    	}
    }
    @Override
    protected Control createDialogArea(Composite parent) {
        Composite container = createClientContainer(parent, 1);
        container.setLayoutData(new GridData(GridData.FILL_BOTH));
        Label nameLabel = new Label(container, SWT.NONE);
        nameLabel.setText(LBL_NAME);
        nameLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL,
                GridData.CENTER, true, true));
        name = new Text(container, SWT.BORDER);
        name.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        name.addListener(SWT.KeyDown, keyLink);
        attrNameDecorator = createDecorator(name);
        name.addModifyListener(updateDecorator);
        if(targetType != AttributeType.LIST && targetType != AttributeType.RESOURCE){

            Label valueLabel = new Label(container, SWT.NONE);
            valueLabel.setText(LBL_VALUE);
            valueLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL,
                    GridData.CENTER, true, true));
            value = new Text(container, SWT.BORDER);
        	value.setLayoutData(new GridData(EditAttrDialog.SINGLE_WIDTH,-1));
        	value.addListener(SWT.KeyDown, keyLink);
        	((Text)value).addModifyListener(valueUpdateDecorator);
        	attrValueDecorator = createDecorator((Text)value);
        	if(initalVal[1] != null)
        		((Text)value).setText(initalVal[1].toString());
    	}else{    		
    		if(targetType == AttributeType.LIST){
    			Composite topPanel = new Composite(container, SWT.NONE);
    			//Flow fl = new FillLayout();

    	        GridLayout gl = new GridLayout(2, false);
    	        gl.marginHeight=gl.marginWidth = 0;
    	        
    			topPanel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    			topPanel.setLayout(gl);
    			
    	        Label valueLabel = new Label(topPanel, SWT.NONE);
    	        valueLabel.setText(LBL_VALUE2);
    	        valueLabel.setLayoutData(new GridData(GridData.BEGINNING,
    	                GridData.CENTER, true, true));
    	        
    	        Composite buttons = new Composite(topPanel, SWT.NONE);
    			//Flow fl = new FillLayout();
    	        buttons.setLayout(gl);
    	        buttons.setLayoutData(new GridData(GridData.END,GridData.CENTER,true,true));
    			
    			Button add = new Button(buttons, SWT.NONE);
    			//add.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    			add.setText("Add");
    			add.addMouseListener(new MouseListener() {
					@Override
					public void mouseUp(MouseEvent e) {
					}
					
					@Override
					public void mouseDown(MouseEvent e) {
						insertAfterSelected();
					}
					
					@Override
					public void mouseDoubleClick(MouseEvent e) {
					}
				});
    			del = new Button(buttons, SWT.NONE);
    			//del.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    			del.setText("Delete");
    			del.setEnabled(false);
    			del.addMouseListener(new MouseListener() {
					@Override
					public void mouseUp(MouseEvent e) {
					}
					
					@Override
					public void mouseDown(MouseEvent e) {
						removeSelected();
					}
					
					@Override
					public void mouseDoubleClick(MouseEvent e) {
					}
				});
    			final Table basetable = new Table(container, SWT.BORDER | SWT.SINGLE | SWT.V_SCROLL);
    			GridData gd = new GridData(GridData.FILL_HORIZONTAL);
    			gd.heightHint = 250;
    			basetable.setLayoutData(gd);
    			table = new TableViewer(basetable);
    			table.setCellEditors(new CellEditor[]{ new TextCellEditor(basetable) });
    			table.setColumnProperties(new String[]{"var"});
    			table.setUseHashlookup(true);
    			
    			TableViewerFocusCellManager focusCellManager = new TableViewerFocusCellManager(table, new FocusCellOwnerDrawHighlighter(table));

    			ColumnViewerEditorActivationStrategy activationSupport = new ColumnViewerEditorActivationStrategy(table) {
    				protected boolean isEditorActivationEvent(ColumnViewerEditorActivationEvent event) {
    					// 	Enable editor only with mouse double click
    					if (event.eventType == ColumnViewerEditorActivationEvent.MOUSE_DOUBLE_CLICK_SELECTION) {
    						EventObject source = event.sourceEvent;
    						if (source instanceof MouseEvent && ((MouseEvent)source).button == 3)
    							return false;
    						return true;
    					}
    					return false;
    				}
    			};

    			table.addSelectionChangedListener(new ISelectionChangedListener() {
					
					@Override
					public void selectionChanged(SelectionChangedEvent event) {
						del.setEnabled(!event.getSelection().isEmpty());
					}
				});
    			
    			TableViewerEditor.create(table, focusCellManager, activationSupport,0);
    			int ops = DND.DROP_MOVE;
    			Transfer[] transfers=new Transfer[]{TextTransfer.getInstance()};
    			table.addDropSupport(ops, transfers, new DropTargetListener() {
					@Override
					public void dropAccept(DropTargetEvent event) {
					}
					
					@Override
					public void drop(DropTargetEvent event) {
						int sel = -1;
						if(event.item != null){
							sel = table.getTable().indexOf((TableItem)event.item) + 1;
							if(sel < 0 || sel >= table.getTable().getItemCount())
								sel = -1;
						}
						String sel2 = (String)event.data;
						if(sel2 != null){
							table.insert(new AttrItem(sel2), sel);
						}
					}
					@Override
					public void dragOver(DropTargetEvent event) {
					}
					
					@Override
					public void dragOperationChanged(DropTargetEvent event) {
					}
					
					@Override
					public void dragLeave(DropTargetEvent event) {
					}
					
					@Override
					public void dragEnter(DropTargetEvent event) {
					}
				});
    			table.addDragSupport(ops, transfers, new DragSourceListener() {
					@Override
					public void dragStart(DragSourceEvent event) {
						event.doit = true;
						sel = table.getTable().getSelectionIndex();
						selval = (AttrItem)(table.getTable().getItem(sel).getData());
						event.data = selval.dataString();
					}
					
					int sel;
					AttrItem selval; 
					
					@Override
					public void dragSetData(DragSourceEvent event) {
						event.data = selval.dataString();
					}
					
					@Override
					public void dragFinished(DragSourceEvent event) {
						if (selval != null)
						{
							if(!(((AttrItem)table.getElementAt(sel)).Id() == selval.Id()))
								sel++;
							table.getTable().remove(sel);
						}
					}
				});
    			//Actions performed when editor is closed.
    			table.setCellModifier(new ICellModifier() {
					@Override
					public void modify(Object element, String property, Object value) {
						TableItem it = (TableItem)element;
						if(value == null || it == null)
							return ;
						it.setText((String)value);
						it.setData(new AttrItem((String)value));
					}
					
					@Override
					public Object getValue(Object element, String property) {
						return element.toString();
					}
					@Override
					public boolean canModify(Object element, String property) {
						return true;
					}
				});
    			ArrayList<String> vals = (ArrayList<String>)initalVal[1];
    			for(String s:vals)
    				table.add(new AttrItem(s));
    			basetable.addKeyListener(new KeyListener() {
					@Override
					public void keyReleased(KeyEvent e) {
					}
					
					@Override
					public void keyPressed(KeyEvent e) {
						if(e.keyCode == SWT.CR)
							okPressed();
						if(e.keyCode == SWT.DEL)
							removeSelected();
						if(e.keyCode == SWT.INSERT)
							insertAfterSelected();
						if(e.keyCode == SWT.UP){
							int ind =table.getTable().getSelectionIndex();
							if(ind>0 && ind<table.getTable().getItemCount()){
								table.getTable().select(ind-1);
								table.getTable().setFocus();
							}else if(table.getTable().getItemCount() > 0){
								table.getTable().select(0);
								table.getTable().setFocus();
							}
						}
						if(e.keyCode == SWT.DOWN){
							int ind =table.getTable().getSelectionIndex();
							if(ind>-1 && ind<(table.getTable().getItemCount()-1)){
								table.getTable().select(ind+1);
								table.getTable().setFocus();
							}else if(table.getTable().getItemCount() > 0){
								table.getTable().select(0);
								table.getTable().setFocus();
							}
						}
					}
				});
    		}
    	}
        name.setText((String)initalVal[0]);

        this.getShell().setText(TITLE);
        this.getShell().setImage(
                PlatformUI.getWorkbench().getSharedImages()
                        .getImage(ISharedImages.IMG_OBJ_ELEMENT));
        return container;
    }
    private void insertAfterSelected(){
    	int ind = table.getTable().getSelectionIndex()+1;
		if(ind >= table.getTable().getItemCount() || ind < 0)
			ind = -1;
		table.insert(new AttrItem(""), ind);
    }
    private void removeSelected(){
    	Table t = table.getTable();
		int sel = t.getSelectionIndex(); 
		if(sel > -1 && sel<t.getItemCount()){
			t.remove(sel);
			t.setFocus();
			if(sel>=t.getItemCount()){
				t.select(sel-1);
				t.setFocus();
			}else{
				t.select(sel);
				t.setFocus();
			}
		}
    }
    private Composite createClientContainer(Composite parent, int span) {
        Composite container = new Composite(parent, SWT.NONE);
        container.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        GridLayout layout = new GridLayout();
        layout.marginWidth = layout.marginHeight = 10;
        layout.verticalSpacing = 2;
        layout.numColumns = span;
        container.setLayout(layout);
        return container;
    }

    
    @Override
    protected void okPressed() {
        String attrname = name.getText().trim();
        curVal[0] = attrname;
        Object toReturn = null;
        if(targetType == AttributeType.RESOURCE){
        	
        }else if (targetType == AttributeType.LIST){
        	ArrayList<String> list = new ArrayList<String>();
        	TableItem[]vals = table.getTable().getItems();
        	int cnt = vals.length;
        	for(int i=0; i<cnt; i++){
        		TableItem s = vals[i];
        		String val = s.getData().toString(); 
        		if(!val.equals("") || i < (cnt-1))
        			list.add(val);
        	}
        	toReturn = list;
        }else{
        	toReturn = ((Text)value).getText();
        }
        curVal[1] = toReturn;
        super.okPressed();
    }

    protected String getNameError(String attrname) {
        if ((initalVal[0].equals(attrname) == false)) {
            for (String a : attrs.keySet()) {
                if (a.equals(attrname) == true && createMode) {
                    return MES_EXIST;
                }
            }
        }
        if (attrname.length() == 0) {
            return MES_EMPTY;
        }
        if (!Attribute.checkNameIsValid(attrname, false))
            return MES_WRONG_FORMAT;

        return null;
    }

    @Override
    protected void createButtonsForButtonBar(Composite parent) {
        createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL,
                false).setEnabled(targetType == AttributeType.RESOURCE || targetType == AttributeType.LIST);
        createButton(parent, IDialogConstants.CANCEL_ID,
                IDialogConstants.CANCEL_LABEL, false);
    }
}