package com.unitesk.requality.eclipse.views.actions;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.Separator;
import org.eclipse.ui.actions.CompoundContributionItem;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.handlers.NewElementHandler;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;

public class NewMenuPoint extends CompoundContributionItem{

	public NewMenuPoint() {
		super();
	}
	
	public NewMenuPoint(String id) {
		super(id);
	}

	@Override
	protected IContributionItem[] getContributionItems() {
		List<TreeNode> sels = RequalityCNF.getSelectedNodes();
		if(sels==null || sels.size()!=1)
			return new IContributionItem[0];
		TreeNode node = RequalityCNF.getSelectedNodes().get(0);
		List<IContributionItem>items = new ArrayList<IContributionItem> ();
		List<IContributionItem> before = new LinkedList<IContributionItem>(),
				after = new LinkedList<IContributionItem>();
		for(NewElementHandler neh : Activator.getDefault().getNewElementHandlers())
			if(neh.isEnabled()){
				String[]types = neh.getElementTypes();
				for(String type:types){
					TreeNode templ = node.getTreeDB().getActiveTemplate(type);
					String label = neh.getShowedElementType(type);
					if(templ!=null){
						String id = templ.getId();
						if(id.length()>17)
							id = id.substring(0,17)+"...";
						label+=" ('"+id+"')";
					}
					ActionContributionItem aci = new ActionContributionItem(new NewTypedTemplatedMenuPoint(node, type, TreeDB.EMPTY_TEMPLATE, neh, label));
					if(aci.isVisible())
						before.add(aci);
					NewTypedMenuPoint mp = new NewTypedMenuPoint(node,neh,type);
					if(mp.isVisible())
						after.add(mp);
				}
			}
		Comparator<IContributionItem> comp = new Comparator<IContributionItem>() {
			@Override
			public int compare(IContributionItem o1, IContributionItem o2) {
				return o1.toString().compareTo(o2.toString());
			}
			
		}; 
		Collections.sort(before, comp);
		Collections.sort(after, comp);
		items.addAll(before);
		if(after.size()>0)
		{
			items.add(new Separator());
			items.addAll(after);
		}
		return items.toArray(new IContributionItem[0]);
	}

}
