/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.views.actions.documents;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.resources.WorkspaceJob;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.documents.IDocumentProcessor;
import com.unitesk.requality.eclipse.DocumentProcessorDescriptor;
import com.unitesk.requality.eclipse.RequalityPlugin;
import com.unitesk.requality.eclipse.editors.NodeEditorInput;
import com.unitesk.requality.eclipse.editors.browser.ReqMarker;
import com.unitesk.requality.eclipse.ui.cnf.CNFAction;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Requirement;

public class DocumentProcessorAction extends CNFAction {
    private DocumentProcessorDescriptor desc;

    public DocumentProcessorAction(DocumentProcessorDescriptor desc,
            ISelectionProvider provider) {
        // TODO:
        // super(provider);
        this.desc = desc;
        setText(desc.getName());
    }

    @Override
    public void run() {
        final TreeDB db = (TreeDB) getTreeDB();
        if(db == null)
            return;
        final List<Document> docs = getSelectedDocuments();
        final IDocumentProcessor proc = desc.getProcessor();
        final WorkspaceJob wj = new WorkspaceJob("Processing "+Arrays.toString(docs.toArray())) {
    		@Override
    			public IStatus runInWorkspace(final IProgressMonitor monitor)
    				throws CoreException {
    					try {
    			            final Requirement rootReq = (Requirement) db.getNode(Requirement
    			                    .getTypeRootQId());
    				        for(final Document doc : docs){
    					        if (!proc.configure(doc)) {
    			                    continue;
    			                }
    					        com.unitesk.requality.documents.IProgressMonitor mon = null;
    							if(monitor!=null)
    								mon = new com.unitesk.requality.documents.IProgressMonitor() {
    									@Override
    									public void worked(int val) {
    										 monitor.worked(val);
    									}
    									
    									@Override
    									public void done() {
    										monitor.done();
    									}
    										
    									@Override
    									public void beginTask(String string, int sumCountersMax) {
    										monitor.beginTask(string, sumCountersMax);
    									}
    									@Override
    									public boolean isCanceled(){
    										return monitor.isCanceled();
    									}
    								};
    							int cnt = 0;
    							proc.process(doc.getQualifiedId(), rootReq, mon);
    							if(mon!=null)
    								mon.done();
    							doc.resetCache();
    							//TODO check
    						}
    				        
    					} catch(IOException ie){
    						ie.printStackTrace();
    						monitor.done();
    						return Status.CANCEL_STATUS;
    					} catch (final Exception e) {
							MessageDialog.openError(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(), "Document Processing Error", e
    		            	   		.getClass().getName() + ": " + e.getMessage());
    		            	e.printStackTrace();
    		            	monitor.done();
    		            	return Status.CANCEL_STATUS;
    		            }
    					PlatformUI.getWorkbench().getDisplay().syncExec(new Runnable() {
		            	    public void run() {
		            	    	for(Document d:docs)
		            	    		if(closeEditor(d)){
		            	    			IWorkbenchPage page = PlatformUI.getWorkbench()
		            	    					.getActiveWorkbenchWindow().getActivePage();
		            	    			if (page != null) {
		            	    				try {
		            	    					page.openEditor(
		            	    							new NodeEditorInput(d),
		            	    							"com.unitesk.requality.editors.browser.ReqMarker",
		            	    							false);
		            	    				} catch (PartInitException e) {
		            	    					e.printStackTrace();
		            	    				}
		            	    			}
		            	    		}
		            	    	monitor.done();
		            	    }
    					});
    					return Status.OK_STATUS;
    				};
            };
         wj.setRule(ResourcesPlugin.getWorkspace().getRoot());
         wj.schedule();
    }

    private boolean closeEditor(Document doc) {
        try {
            IEditorReference eRef = RequalityPlugin.findMarkEditorForDocument(doc
                    .getQualifiedId());
            if (eRef == null)
                return false;
            ((ReqMarker) eRef.getEditor(false)).close(false);
            return true;
        } catch (PartInitException e) {
            e.printStackTrace();
        }
        return false;
    }

    private TreeDB getTreeDB() {
        List<Document> docs = getSelectedDocuments();

        if (docs != null) {
            return docs.get(0).getTreeDB();
        }
        return null;
    }

    @Override
    public boolean isEnabled() {
        return getSelectedDocuments() != null;
    }

    private List<Document> getSelectedDocuments() {
        List<Document> docs = new ArrayList<Document>();
        List<TreeNode> nodes = RequalityCNF.getSelectedNodes();
        TreeDB db = null;
        if(nodes == null)
            return null;
        for(TreeNode node : nodes)
            if (node instanceof Document) {
                docs.add((Document)node);
                if(db == null)
                    db = node.getTreeDB();
                if(db != node.getTreeDB())
                    return null;
            }else{
                return null;
            }
        if(docs.size()==0)
            return null;
        return docs;
    }
}
