/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.views.documents;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.UUID;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.nodetypes.ToDoItem;
import com.unitesk.requality.tools.DeepFirstTreeWalker;


/**
 * Class representing a list of tasks for tasks list after update processor 
 */
public class TaskList {

	private ArrayList<Task> tasks;
	private Set<ITaskListViewer> changeListeners;
	
	TreeDB db;
	
	public TaskList() {
		tasks = new ArrayList<Task>();
		changeListeners = new HashSet<ITaskListViewer>();
	}
	
	
	public TaskList(TreeDB db) {
		
		this.db = db;
		
		tasks = new ArrayList<Task>();
		changeListeners = new HashSet<ITaskListViewer>();
		
		ArrayList<Requirement> reqsToDoList = new ArrayList<Requirement>();
		
		for (TreeNode node : new DeepFirstTreeWalker(db.getRootNode())) {
			
			
			if (node.getType().equals(ToDoItem.TYPE_NAME)) {
				
				Object attr_req_link = UUID.fromString(node.getAttributeValue(ToDoItem.ATTR_REQ_LINK).toString());
				
				if (attr_req_link instanceof java.util.UUID) {
					UUID reqUUID = (UUID) attr_req_link;
					TreeNode parent = db.getNode(reqUUID);
					if (parent != null) {
						if (parent.getType().equals(Requirement.TYPE_NAME)) {
							Requirement req = (Requirement)parent;
							
							reqsToDoList.add(req);
							
							
							String oldDocQID = node.getAttributeValue(ToDoItem.OLD_DOC).toString();
							String newDocQID = node.getAttributeValue(ToDoItem.NEW_DOC).toString();
							
							Document oldDoc = (Document) db.getNode(oldDocQID);
							Document newDoc = (Document) db.getNode(newDocQID);
							
							if ((oldDoc == null) || (newDoc) == null) {
								continue;
							}
							
							String status = node.getStringAttributeValue(ToDoItem.ATTR_STATUS);

							
							Task task;
							
							if (status == null) {
							
								task = new Task(
									req, Task.Action.valueOf( node.getAttributeValue(ToDoItem.ATTR_TRANSF_ACTION).toString() ), 
									oldDocQID, newDocQID, node.getUUId() 
									);
							} else {
								task = new Task(
										req, Task.Action.valueOf( node.getAttributeValue(ToDoItem.ATTR_TRANSF_ACTION).toString() ), 
										oldDocQID, newDocQID, node.getUUId(), Task.Status.valueOf(status) 
										);
							}
							tasks.add(task);
						}
					}
				} else {
					System.out.println("=Cannot cast <" + attr_req_link + "> to UUID");
				}
			}
		}
		
	}
	
	public TaskList(ArrayList<Task> tasks) {
		this.tasks = tasks;
		changeListeners = new HashSet<ITaskListViewer>();
	}

	
	public ArrayList<Task> getTasks() {
		return tasks;
	}
	
	public void taskChanged(Task task) {
		Iterator<ITaskListViewer> iterator = changeListeners.iterator();
		while (iterator.hasNext())
			iterator.next().updateTask(task);
	}

	public void removeChangeListener(ITaskListViewer viewer) {
		changeListeners.remove(viewer);
	}

	public void addChangeListener(ITaskListViewer viewer) {
		changeListeners.add(viewer);
	}


	public Set<String> getListOfDocQIds() {
		Set<String> docQIdsSet = new HashSet<String>();
		for (Task task : tasks) {
			String oldDocQId = task.getOldDocQID();
			Document oldDoc = (Document) db.getNode(oldDocQId);
			
			
			if (oldDoc != null) {
				docQIdsSet.add(oldDocQId.substring(oldDocQId.lastIndexOf("/") + 1));
			}
			String newDocQId = task.getNewDocQID();
			Document newDoc = (Document) db.getNode(newDocQId);
			if (newDoc != null) {
				docQIdsSet.add(oldDocQId.substring(newDocQId.lastIndexOf("/") + 1));
			}
			
		}
		
		return docQIdsSet;
	}

}
