/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.views.documents;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.resources.IProject;
import org.eclipse.e4.ui.model.application.ui.MElementContainer;
import org.eclipse.e4.ui.model.application.ui.MUIElement;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.basic.MPartSashContainerElement;
import org.eclipse.e4.ui.model.application.ui.basic.impl.PartStackImpl;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.part.ViewPart;

import com.unitesk.requality.core.ITreeChangeListener;
import com.unitesk.requality.core.NodeDesc;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.core.TreeNodeChangeEvent;
import com.unitesk.requality.core.attribute.Attribute;
import com.unitesk.requality.core.attribute.AttributeType;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.core.ITreeStateListener;
import com.unitesk.requality.eclipse.core.IUIStateChangeListener;
import com.unitesk.requality.eclipse.core.TreesTracker;
import com.unitesk.requality.eclipse.editors.NodeEditorInput;
import com.unitesk.requality.eclipse.editors.browser.ReqMarker;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.nodetypes.ToDoItem;

/**
 * A class for processing a view of tasks list after update processor
 */
@SuppressWarnings("restriction")
public class ToDoList extends ViewPart implements ITreeChangeListener, ITreeStateListener {
	volatile TableViewer tableViewer;
	Table table;
	private Button closeButton;
	private boolean isDisposed = false;
	
	private static boolean hideCompleted = false;

	private static List<ToDoList> activeToDoViews = new ArrayList<ToDoList>();
	
	public static boolean isHideCompleted() {
		return hideCompleted;
		
	}

	public static void setHideCompleted(boolean hideCompleted) {
		ToDoList.hideCompleted = hideCompleted;
		
		for(ToDoList toDoView : activeToDoViews) {
			toDoView.refresh();
		}
	}

	private TreeDB db;
	
	private boolean isModified = false;
	
	static ToDoItem selectedItem = null;

	Document oldDoc;
	Document newDoc;
	private volatile TaskList taskList = null;

	public static final String ID = "com.unitesk.requality.views.ToDoList";

	public ToDoList() {
		activeToDoViews.add(this);
		isDisposed = false;
	}

	public void init(IViewSite site) throws PartInitException {
		super.init(site);
	}
	
	private void initElements(List<TreeNode> nodes) {
		if (isDisposed) {
			return;
		}

		if (nodes.size() > 0) {
			taskList = null;
			try {
				if(db != null) {
					db.removeGUIListener(this);
				}
				db = nodes.get(0).getTreeDB();
				db.addGUIListener(this);
				taskList = new TaskList(db);
				
				
			} catch (NullPointerException e) {
				taskList = new TaskList();
			} finally {
				if (taskList == null) {
					taskList = new TaskList();
				}
				
				setInputInNewThread();
			}

		}
	}

	public void createPartControl(Composite p1) {
		Composite parent = new Composite(p1, SWT.NONE);
		parent.setLayout(new GridLayout(1,true));
		Activator.getDefault().addChangeListener(new IUIStateChangeListener() {

			@Override
			public void newActiveNodes(List<TreeNode> nodes) {
				// TODO listener of CNF
				// List<Object> ar = RequalityCNF.getSelectedObj();
				// if(ar.size()>0)
				// ar.get(0);

				initElements(nodes);
			}

			
		});

		int style = SWT.SINGLE | SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL
				| SWT.FULL_SELECTION | SWT.HIDE_SELECTION;

		//parent.setLayout(new GridLayout(1, true));
		table = new Table(parent, style);

		GridData gridData = new GridData(GridData.FILL_BOTH);
		gridData.grabExcessVerticalSpace = true;
		gridData.horizontalSpan = 3;
		table.setLayoutData(gridData);

		table.setLinesVisible(true);
		table.setHeaderVisible(true);
		
		tableViewer = new TableViewer(table);
		
		tableViewer.setSorter(new TaskSorter());

		TableColumn column = new TableColumn(table, SWT.CENTER, 0);
		column.setText(" ");
		column.setWidth(20);
		// by clicking on cell above picture - sort by locations
		column.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				((TaskSorter) tableViewer.getSorter()).doSort(TaskSorter.PICTURE);
				tableViewer.refresh();
			}
		});
		

		column = new TableColumn(table, SWT.LEFT, 1);
		column.setText("Requirement Description");
		column.setWidth(400);
		column.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected(SelectionEvent e) {
				((TaskSorter) tableViewer.getSorter()).doSort(TaskSorter.DESCRIPTION);
				tableViewer.refresh();
			}
		});

		column = new TableColumn(table, SWT.LEFT, 2);
		column.setText("Action");
		column.setWidth(100);
		column.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				((TaskSorter) tableViewer.getSorter()).doSort(TaskSorter.ACTION);
				tableViewer.refresh();
			}
		});
		

		column = new TableColumn(table, SWT.LEFT, 3);
		column.setText("Status");
		column.setWidth(100);
		column.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				((TaskSorter) tableViewer.getSorter()).doSort(TaskSorter.STATUS);
				tableViewer.refresh();
			}
		});
		
		
		
		tableViewer.setUseHashlookup(true);

		tableViewer.setContentProvider(new ToDoListContentProvider());
		tableViewer.setLabelProvider(new TaskLabelProvider());

		try {

			TreeDB db = RequalityCNF.getSelectedNodes().get(0).getTreeDB();
			if (taskList == null) {
				taskList = new TaskList(db);
			}
			setInputInNewThread();
		} catch (NullPointerException e) {
			taskList = new TaskList();
			setInputInNewThread();
		}

		Composite buttons = new Composite(parent, SWT.NO_SCROLL);
		buttons.setLayout(new GridLayout(5, true));

		GridData gridDataButtons = new GridData(GridData.FILL_HORIZONTAL);
		buttons.setLayoutData(gridDataButtons);

		createButtons(buttons);

		tableViewer.addDoubleClickListener(new IDoubleClickListener() {

			@Override
			public void doubleClick(DoubleClickEvent event) {
				
				IStructuredSelection selection = (IStructuredSelection) tableViewer
						.getSelection();
				
				if (selection.getFirstElement() instanceof Task) {
					
					Task task = (Task) selection.getFirstElement();
					
					Requirement req = task.getReq();
					
					Location loc = null;
					
					String oldDocID = task.getOldDocQID();
					
					NodeEditorInput input = null;
					
					for (Location oldLoc : req.getLocations()) {
			        	if (oldLoc.getParent().getQualifiedId().equals(oldDocID)) {
			        		loc = oldLoc;
			        		
			        		input = new NodeEditorInput(loc.getParent());
			        		input.setData(loc);
			        		break;
			        	}
			        }
					
					
				    
					Location locInNewDoc = null;
			        String newDocID = task.getNewDocQID();
			        
			        NodeEditorInput inputInNewDoc = null;
				    
			        for (Location newLoc : req.getLocations()) {
			        	if (newLoc.getParent().getQualifiedId().equals(newDocID)) {
			        		locInNewDoc = newLoc;
			        		
			        		inputInNewDoc = new NodeEditorInput(locInNewDoc.getParent());
			        		inputInNewDoc.setData(locInNewDoc);
			        		break;
			        	}
			        }
			        
			        if (locInNewDoc == null) {
//			        	locInNewDoc = (Location) db.getNode(task.getNewDocQID());
			        	inputInNewDoc = new NodeEditorInput(db.getNode(task.getNewDocQID()));
//			        	inputInNewDoc.setData(locInNewDoc);
			        }
			        
			        
			        
			        
					try {
						
						if ((db.getNode(oldDocID) == null) || (db.getNode(newDocID) == null) ) {
							refresh();
							showMessageBox("Update Processor Tasks", "One of the documents has been removed or deleted.");
							return;
						}
						
						
						
//						if (inputInNewDoc == null && (db.getNode(task.getNewDocQID()) != null) ) {
//				        	inputInNewDoc = new NodeEditorInput(db.getNode(task.getNewDocQID()));
//				        }
						
						if ((input == null) || (inputInNewDoc == null)) {
							refresh();
							showMessageBox("Update Processor Tasks", "One of the documents has been removed or deleted.");
							return;
						}
						
						IWorkbenchPage p = PlatformUI.getWorkbench()
								.getActiveWorkbenchWindow().getActivePage();
						IEditorPart ip = IDE.openEditor(p, input, ReqMarker.ID);
						((ReqMarker)ip).setTargetLocation(loc);

						EModelService modelService = (EModelService) ip
								.getSite().getService(EModelService.class);
						MPartSashContainerElement mpartService = (MPart) ip
								.getSite().getService(MPart.class);

						MPartSashContainerElement cn = (MPartSashContainerElement) mpartService
								.getParent();
						MPartSashContainerElement cn2 = null;
						boolean changed = false;

						for (MUIElement muie : cn.getParent().getChildren())
							if (muie instanceof MElementContainer<?>) {
								for (MUIElement mu : ((MElementContainer<MUIElement>) muie)
										.getChildren())
									if (mu.getTags().contains("Editor")
											&& !cn.equals(muie))
										cn2 = (MPartSashContainerElement) muie;

							}
						
						

						if (cn2 == null) {
							changed = true;
							cn2 = (MPartSashContainerElement) modelService
									.cloneElement(cn, null);
						}
						
						
						
//						IEditorPart ip2 = IDE.openEditor(PlatformUI
//								.getWorkbench().getActiveWorkbenchWindow()
//								.getActivePage(),
//								inputInNewDoc, ReqMarker.ID);
						
						IEditorPart ip2 = IDE.openEditor(p,
								inputInNewDoc, ReqMarker.ID);
						
			            // if locInNewDoc is null this will clear the highlighting
						((ReqMarker)ip2).setTargetLocation(locInNewDoc);
						
						mpartService = (MPart) ip2.getSite().getService(
								MPart.class);
						

						boolean containsMpartService = false;
						if (changed) {
							((PartStackImpl) cn2).getChildren().clear();
						} else {
							containsMpartService = ((PartStackImpl) cn2).getChildren().contains((MPart) mpartService);
						}
						
						if ( !containsMpartService) {
							((PartStackImpl) cn2).getChildren().add(
								(MPart) mpartService);
							
						}
							
						if (changed) {
							modelService.insert(cn2, cn, EModelService.RIGHT_OF, 0.5f);
						}
						
						modelService.bringToTop(mpartService);
						modelService.bringToTop((MPart) ip.getSite().getService(
								MPart.class));
						
			            
			        } catch (PartInitException e) {
			            e.printStackTrace();
			        } 
					
					Map<String, List<TreeNode> > m = new HashMap<String, List<TreeNode> >();
					m.put(req.getType(), Arrays.asList(new TreeNode[]{req}));
					Activator.getDefault().setActiveNodes(m);
					
				} else {
					return;
				}
			}
		});
		
		tableViewer.addSelectionChangedListener(new ISelectionChangedListener() {
			
			@Override
			public void selectionChanged(SelectionChangedEvent arg0) {
				
				if (isModified) {
					refresh();
					isModified = false;
				}
				
				Task currentNode = (Task)((StructuredSelection)tableViewer.getSelection()).getFirstElement();
				if (currentNode != null) {
					if (db == null) {
						db = currentNode.getReq().getTreeDB();
					} else {
						db.removeGUIListener(ToDoList.this);
						db = currentNode.getReq().getTreeDB();
					}
					db.addGUIListener(ToDoList.this);
					TreeNode treeNode = db.getNode(currentNode.getToDoItemUUID());
					selectedItem = (ToDoItem)treeNode;
				}
			}
		});
		
		TreesTracker.getTracker().addListener(this); 
	}

	public void setFocus() {
		tableViewer.getControl().setFocus();
	}

	public void dispose() {
		tableViewer.getLabelProvider().dispose();
		Activator.getDefault().removeChangeListener(null);
		isDisposed = true;
		activeToDoViews.remove(this);
		TreesTracker.getTracker().removeListener(this);
	}

	class ToDoListContentProvider implements IStructuredContentProvider,
			ITaskListViewer {

		public void inputChanged(Viewer v, Object oldInput, Object newInput) {
			if (newInput != null)
				((TaskList) newInput).addChangeListener(this);
			if (oldInput != null)
				((TaskList) oldInput).removeChangeListener(this);

		}

		public void dispose() {
		}

		public Object[] getElements(Object parentElement) {
			
			if (hideCompleted) {
				ArrayList<Task> allTasks = ((TaskList) parentElement).getTasks();
				ArrayList<Task> tasksToCheck = new ArrayList<Task>();
				for (Task task : allTasks) {
					if (task.getStatus().equals(Task.Status.CHECK)) {
						tasksToCheck.add(task);
					}
				}
				return tasksToCheck.toArray();
			} else {
				return ((TaskList) parentElement).getTasks().toArray();
			}

		}

		public void addTask(Task task) {
			tableViewer.add(task);
		}

		public void removeTask(Task task) {
			tableViewer.remove(task);
		}

		public void updateTask(Task task) {
			tableViewer.update(task, null);
		}
	}

	private void createButtons(Composite parent) {
		Button check = new Button(parent, SWT.PUSH);
		check.setText("Check");
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING);
		gridData.widthHint = 80;
		check.setLayoutData(gridData);

		check.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected(SelectionEvent e) {
				Task task = (Task) ((IStructuredSelection) tableViewer
						.getSelection()).getFirstElement();
				if (task != null) {
					task.setStatus(Task.Status.CHECK);
					TreeNode item = db.getNode(task.getToDoItemUUID());
					item.putAttribute(new Attribute(item, AttributeType.STRING,
							ToDoItem.ATTR_STATUS, Task.Status.CHECK));
					item.saveAttributes();
					tableViewer.refresh();
				}
			}
		});

		Button complete = new Button(parent, SWT.PUSH);
		complete.setText("Complete");
		gridData = new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING);
		gridData.widthHint = 80;
		complete.setLayoutData(gridData);

		complete.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected(SelectionEvent e) {
				Task task = (Task) ((IStructuredSelection) tableViewer
						.getSelection()).getFirstElement();
				if (task != null) {
					task.setStatus(Task.Status.COMPLETED);
					TreeNode item = db.getNode(task.getToDoItemUUID());
					item.putAttribute(new Attribute(item, AttributeType.STRING,
							ToDoItem.ATTR_STATUS, Task.Status.COMPLETED));
					item.saveAttributes();
					tableViewer.refresh();
				}
			}
		});

	}
	
	public void refresh(TreeDB db) {
		
		try {
			if(db != null) {
				taskList = new TaskList(db);
			}
		} catch (NullPointerException e) {
			taskList = new TaskList();
		} finally {
			if (taskList == null) {
				taskList = new TaskList();
			}
			setInputInNewThread();
		}
		setInputInNewThread();
	
	}

	public ISelection getSelection() {
		return tableViewer.getSelection();
	}

	public Control getControl() {
		return table.getParent();
	}

	public Button getCloseButton() {
		return closeButton;
	}
	
	public static ToDoItem getSelectedNode() {
		return selectedItem;
	}

	@Override
	public void created(TreeNode node, boolean internal) {
		isModified = true;
		
		if (Document.TYPE_NAME.equals(node.getType()) && 
				taskList.getListOfDocQIds().
					contains(node.getId().substring(node.getId().lastIndexOf("/") + 1))) {
			refresh();
		}
	}


	@Override
	public void deleted(NodeDesc nodeDesc, boolean internal) {
		for (TableItem ti: tableViewer.getTable().getItems()){
			Task task = ((Task)ti.getData());
			if (task.getToDoItemUUID().equals(nodeDesc.getUUId())) {
				tableViewer.remove(task);
				taskList.getTasks().remove(task);
			}
		}
		
		if (Document.TYPE_NAME.equals(nodeDesc.getType()) && 
				taskList.getListOfDocQIds().
					contains(nodeDesc.getId().substring(nodeDesc.getId().lastIndexOf("/") + 1))) {
			refresh();
		}
		
	}

	@Override
	public void moved(TreeNode node, TreeNode oldParent, boolean internal) {
		// TODO Auto-generated method stub
		
	}
	
	
	private void refresh() {
		if (isDisposed) {
			return;
		}

		if (db != null) {
			
			taskList = null;
			try {
				taskList = new TaskList(db);
			} catch (NullPointerException e) {
				taskList = new TaskList();
			} finally {
				if (taskList == null) {
					taskList = new TaskList();
				}
				setInputInNewThread();
			}

		} else {
			taskList = new TaskList();
			setInputInNewThread();
		}
	}



	private void setInputInNewThread() {
		Display.getDefault().asyncExec(new Runnable() {
		       public void run() {
		          tableViewer.setInput(taskList);
		       }
		});
	}
	
	
	protected void showMessageBox(final String messageBoxTitle,
			final String messageBoxMessage) {
		IWorkbench w = PlatformUI.getWorkbench();
		w.getDisplay().syncExec(new Runnable() {
			public void run() {
				Shell activeShell = PlatformUI.getWorkbench()
						.getActiveWorkbenchWindow().getShell();
				MessageDialog.openInformation(activeShell, messageBoxTitle,
						messageBoxMessage);
			}
		});

	}



	@Override
	public void open(TreeDB db) {
		this.db = db;
		refresh(db);
		
	}


	@Override
	public void close(TreeDB db) {
		this.db = null;
		refresh();
		
	}



	@Override
	public void delete(IProject proj) {
		this.db = null;
		refresh();
		
		
	}

	@Override
	public void changed(TreeNode node, Set<String> params, boolean internal) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void changed(List<TreeNodeChangeEvent> changes) {
		for(TreeNodeChangeEvent ev:changes)
			switch (ev.getType()) {
			case CREATE:
				created(ev.getTreeNodeTarget(), (Boolean)ev.getArguments()[0]);
				break;
			case DELETE:
				deleted(ev.getNodeDescTarget(), (Boolean)ev.getArguments()[0]);
				break;
			default:
				break;
			}
	}
	
}
