/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.views.properties;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPartSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.model.WorkbenchLabelProvider;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;

import com.unitesk.requality.core.ITreeChangeListener;
import com.unitesk.requality.core.NodeDesc;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.core.TreeNodeChangeEvent;
import com.unitesk.requality.core.TreeNodeChangeEvent.TreeNodeChangeEventType;
import com.unitesk.requality.eclipse.RequalityPlugin;
import com.unitesk.requality.eclipse.editors.NodeEditorInput;
import com.unitesk.requality.eclipse.editors.browser.ReqMarker;
import com.unitesk.requality.eclipse.handlers.location.ILocationTab;
import com.unitesk.requality.eclipse.ui.cnf.RequalitySorter;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.nodetypes.TextNode;
import com.unitesk.requality.nodetypes.VirtualNode;

public class AltDescriptionPropertySection extends TreeNodePropertiesTab implements ILocationTab{

    private ElementDescriptionPanel descr;
    private TreeViewer locations;
    private Requirement req;
    private Composite par,locholder; 
    private Label descLabel;

    public AltDescriptionPropertySection() {
    }

    public ITreeChangeListener tcl = new ITreeChangeListener() {
    	private boolean relates(TreeNode node){
    		return node.getUUId().equals(req.getUUId())|| req.getLocations().contains(node);
    	}
    	private boolean relates(NodeDesc node){
    		if (node.getUUId().equals(req.getUUId()))
    			return true;
    		//TODO check
    		for(Location l:req.getLocations())
    			if(l.getUUId().equals(node.getUUId()))
    				return true;
    		return false;
    	}
		@Override
		public void moved(TreeNode node, TreeNode oldParent, boolean internal) {
			if(isDisposed){
	    		if(req!=null)
	    			req.getTreeDB().removeListener(tcl);
	    		return;
	    	}
			//TODO needed?
			if(relates(node.getParent()))
				updateNode(node.getParent());
			if(relates(oldParent) && !node.getParent().getUUId().equals(oldParent.getUUId()))
				updateNode(oldParent);
			if(relates(node))
				updateNode(node);
		}
		
		@Override
		public void deleted(final NodeDesc nodeDesc, boolean internal) {
			if(isDisposed){
	    		if(req!=null)
	    			req.getTreeDB().removeListener(tcl);
	    		return;
	    	}
			if(!relates(nodeDesc))
				return;
			PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
        	    public void run() {
					if(nodeDesc.getType().equals(Requirement.TYPE_NAME) && req.getUUId().equals(nodeDesc.getUUId()))
						locations.refresh();
					if(nodeDesc.getType().equals(Location.TYPE_NAME)){
						if(req.getLocations().contains(nodeDesc.getUUId()))
							locations.refresh();
					}
        	    }
			});
		}
		
		@Override
		public void created(TreeNode node, boolean internal) {
			if(isDisposed){
	    		if(req!=null)
	    			req.getTreeDB().removeListener(tcl);
	    		return;
	    	}
			if(!relates(node))
				return;
			if(req.getLocations().contains(node))
				updateNode(node);
		}
		
		@Override
		public void changed(TreeNode node, Set<String> attrs, boolean internal) {
			if(isDisposed){
	    		if(req!=null)
	    			req.getTreeDB().removeListener(tcl);
	    		return;
	    	}
			if(attrs.contains(TreeNode.ATTR_DESCRIPTION)||attrs.contains(Requirement.ATTR_LOCATIONS))
				updateNode(node);
		}
		private void updateNode(final TreeNode node){
			PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
        	    public void run() {
        	    	if(isDisposed){
        	    		if(req!=null)
        	    			req.getTreeDB().removeListener(tcl);
        	    		return;
        	    	}
					if(node.getType().equals(Requirement.TYPE_NAME) && req.getUUId().equals(node))
						locations.refresh(node,true);
					if(node.getType().equals(Location.TYPE_NAME)){
						if(req.getLocationQIds().contains(node.getUUId()))
							locations.refresh(req,true);
					}
        	    }
			});
		}

		@Override
		public void changed(List<TreeNodeChangeEvent> changes) {
			int cnt = 0;
			for(TreeNodeChangeEvent ch:changes){
				TreeNode node = ch.getTreeNodeTarget();
				if(node!=null && relates(node)){
					cnt++;
					/*switch (ch.getType()) {
						case TreeNodeChangeEventType.CHANGE:
							break;

						case TreeNodeChangeEventType.CREATE:
							break;

						case TreeNodeChangeEventType.MOVE:
							break;
					}*/
				}
				NodeDesc nddc = ch.getNodeDescTarget();
				if(nddc!=null && relates(nddc)){
					if(ch.getType() == TreeNodeChangeEventType.DELETE)
						cnt++;
						//deleted(nddc, (Boolean)ch.getArguments()[0]);
				}
			}
			if(cnt>2)
				refresh();
			else{
				for(TreeNodeChangeEvent ch:changes){
					TreeNode node = ch.getTreeNodeTarget();
					if(node!=null && relates(node)){
						switch (ch.getType()) {
							case CHANGE:
								changed(node,(Set<String>)ch.getArguments()[1],(Boolean)ch.getArguments()[0]);
								break;

							case CREATE:
								created(node, (Boolean)ch.getArguments()[0]);
								break;

							case MOVE:
								//addGuiChange(new TreeNodeChangeEvent(TreeNodeChangeEventType.MOVE, node, internal, oldParent == null ? node.getParent(true) : oldParent));
								moved(node,(TreeNode) ch.getArguments()[1], (Boolean)ch.getArguments()[0]);
								break;
							default:
						}
					}else
						if(node !=null && ch.getType()==TreeNodeChangeEventType.MOVE){
							//TODO Maybe microreduction here to be made
							if(relates(node.getParent()) || relates((TreeNode) ch.getArguments()[1]))
								moved(node,(TreeNode) ch.getArguments()[1], (Boolean)ch.getArguments()[0]);
						}
					NodeDesc nddc = ch.getNodeDescTarget();
					if(nddc!=null && relates(nddc)){
						if(ch.getType() == TreeNodeChangeEventType.DELETE)
							deleted(nddc, (Boolean)ch.getArguments()[0]);
					}
				}
			}
		}
	};
    @Override
    public void createControls(Composite parent,
            TabbedPropertySheetPage aTabbedPropertySheetPage) {
        super.createControls(parent, aTabbedPropertySheetPage);
        TabbedPropertySheetWidgetFactory wf = aTabbedPropertySheetPage
                .getWidgetFactory();
        parent.setLayout(new GridLayout());
        parent.setLayoutData(new GridData(GridData.FILL_BOTH));
        
        descLabel = wf.createLabel(parent, "Alternative description:");
        
        descr = new ElementDescriptionPanel(parent, SWT.MULTI | SWT.BORDER | SWT.H_SCROLL
                | SWT.V_SCROLL);
        descr.getControl().setLayoutData(new GridData(GridData.FILL_BOTH));
        descr.getControl().addFocusListener(getDescriptionFocusAdapter(descr.getDescriptionControl(), node));
        
        par = parent;
        locholder = new Composite(parent, SWT.NONE);
        locholder.setLayout(new GridLayout(1,true));
        locholder.setLayoutData(new GridData(GridData.FILL_BOTH));
        locholder.setBackground(parent.getBackground());
        	wf.createLabel(locholder, "Locations:");
	        locations = new TreeViewer(locholder, SWT.BORDER | SWT.H_SCROLL
	                | SWT.V_SCROLL | SWT.MULTI);
	        locations.getControl().setLayoutData(new GridData(GridData.FILL_BOTH));
	        locations.setLabelProvider(WorkbenchLabelProvider
	                .getDecoratingWorkbenchLabelProvider());
	        locations.setSorter(new RequalitySorter());
	        locations.setContentProvider(new ITreeContentProvider() {
	            @Override
	            public void inputChanged(Viewer viewer, Object oldInput,
	                    Object newInput) {
	            }
	
	            @Override
	            public void dispose() {
	            }
	
	            @Override
	            public boolean hasChildren(Object element) {
	                TreeNode node = (TreeNode) element;
	                return node.sizeChildren() > 0;
	            }
	
	            @Override
	            public Object getParent(Object element) {
	                return ((TreeNode) element).getParent();
	            }
	
	            @Override
	            public Object[] getElements(Object inputElement) {
	                return (Object[]) inputElement;
	            }
	
	            @Override
	            public Object[] getChildren(Object parentElement) {
	                TreeNode node = (TreeNode) parentElement;
	                ArrayList<Object> result = new ArrayList<Object>();
	                if (node.getType().equals(Document.TYPE_NAME)) {
	                    if(req != null)
	                        for (int i = 0; i < node.sizeChildren(); i++) {
	                            Location loc = (Location) node.getChild(i);
	                            if (Arrays.asList(loc.getRequirements()).contains(req)) {
	                                result.add(loc);
	                            }
	                        }
	                }
	                return result.toArray();
	            }
	        });
	        locations.addDoubleClickListener(new IDoubleClickListener() {
	
	            @Override
	            public void doubleClick(DoubleClickEvent event) {
	                IStructuredSelection selection = (IStructuredSelection) locations
	                        .getSelection();
	                NodeEditorInput input;
	                if (selection.getFirstElement() instanceof Document) {
	                    Document doc = (Document) selection.getFirstElement();
	                    input = new NodeEditorInput(doc);
	                } else if (selection.getFirstElement() instanceof Location) {
	                    Location loc = (Location) selection.getFirstElement();
	                    input = new NodeEditorInput(loc.getParent());
	                    input.setData(loc);
	                } else {
	                    return;
	                }
	
	                try {
	                    String docId = input.getStorage().getNode()
	                            .getQualifiedId();
	                    TreeDB db = node.getTreeDB();
	                    if(db.isVirtual())
	                    	db = VirtualNode.getBaseTree(db);
	                    if(db == null)
	                    	return;
	                    IEditorReference ref = RequalityPlugin
	                            .findMarkEditorForDocument(
	                                    docId);
	                    if (ref != null && ref.getEditor(true) != null) {
	                        ReqMarker ed = (ReqMarker) ref.getEditor(true);
	                        ed.setTargetLocation((Location) input.getData());
	                    }
	                    IDE.openEditor(PlatformUI.getWorkbench()
	                            .getActiveWorkbenchWindow().getActivePage(), input,
	                            ReqMarker.ID, false);
	                } catch (PartInitException e) {
	                    e.printStackTrace();
	                } catch (CoreException e) {
	                    e.printStackTrace();
	                }
	            }
	        });
        IViewPart view = PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow().getActivePage()
                .findView("org.eclipse.ui.views.PropertySheet");
        if (view == null)
            return;
        IWorkbenchPartSite site = view.getSite();
        MenuManager mgr = new MenuManager();
        mgr.setRemoveAllWhenShown(true);
        Menu menu = mgr.createContextMenu(locations.getControl());
        locations.getControl().setMenu(menu);
        site.registerContextMenu(mgr, site.getSelectionProvider());
    }

    public void refreshLocations(){
        setTreeNode(req);
    }

    @Override
    public void dispose() {
        super.dispose();
    }

    @Override
    protected void setTreeNode(TreeNode node) {
    	if(req!=null)
    		req.getTreeDB().removeGUIListener(tcl);
        req = toRequirement(node);
        if (req == null || locations == null) {
            return;
        }
        descLabel.setText((req.getType().equals(Requirement.TYPE_NAME)?"Alternative d":"D")+ "escription:");
        descr.setDocument(new org.eclipse.jface.text.Document(req
                .getDescription()));
        Set<com.unitesk.requality.nodetypes.Document> docs = new HashSet<com.unitesk.requality.nodetypes.Document>();
        for (Location l : req.getLocations()) {
            if (!docs.contains(l.getParent())) {
                docs.add((com.unitesk.requality.nodetypes.Document) l
                        .getParent());
            }
        }
        if(!req.getType().equals(TextNode.TYPE_NAME))
        {
        	locholder.setParent(par);
        	locholder.setVisible(true);
        	locholder.setLayoutData(new GridData(GridData.FILL_BOTH));
        	par.layout();
        	locations.setInput(docs.toArray());
        	locations.expandAll();
        }else{
        	locholder.setVisible(false);
        	locholder.setLayoutData(new GridData(0,0));
        	locholder.getParent().layout();
        }
        if(req!=null)
    		req.getTreeDB().addGUIListener(tcl);
    }
    @Override 
    public void aboutToBeHidden() {
    	if(req!=null)
    		req.getTreeDB().addGUIListener(tcl);
    };
	@Override
	public Map<Location, List<Requirement>> getLocationsAndRequirements() {
		Map<Location, List<Requirement>> map = new HashMap<Location, List<Requirement>>(); 
		ISelection sel = locations.getSelection();
		 if (!locations.getControl().isFocusControl() || req == null ||
	               !(sel instanceof IStructuredSelection)) {
	         return null;
	     }
	     IStructuredSelection ssel = (IStructuredSelection) sel;
	     java.util.Iterator it = ssel.iterator();
	     List<Location> toremove = new ArrayList<Location>();
	     while(it.hasNext())
	     {
	      	Object o = it.next();
	      	if(o instanceof Location){
	      		if(!toremove.contains(o))
	      			toremove.add((Location)o);
	      	}else
	      	{
	      		toremove.addAll(req.getLocations());
	      	}
	     }
	     for(Location l : toremove){
	      	if(!map.containsKey(l))
	      		map.put(l, new ArrayList<Requirement>());
	      	List<Requirement> list = map.get(l);
	      	if(!list.contains(req))
	      		list.add(req);
	       	map.put(l, list);
   		 }
	     return map.size()>0?map:null;
	}

}
