/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.views.properties;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import java.util.UUID;

import org.eclipse.core.resources.WorkspaceJob;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.fieldassist.ControlDecoration;
import org.eclipse.jface.fieldassist.FieldDecoration;
import org.eclipse.jface.fieldassist.FieldDecorationRegistry;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ColumnViewerEditorActivationEvent;
import org.eclipse.jface.viewers.ColumnViewerEditorActivationListener;
import org.eclipse.jface.viewers.ColumnViewerEditorDeactivationEvent;
import org.eclipse.jface.viewers.ColumnViewerToolTipSupport;
import org.eclipse.jface.viewers.ComboBoxCellEditor;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.ICellEditorValidator;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableColorProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.jface.window.DefaultToolTip;
import org.eclipse.jface.window.ToolTip;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.events.MouseTrackListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.ScrollBar;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.progress.UIJob;

import com.unitesk.requality.core.IAttributeFilter;
import com.unitesk.requality.core.INodeChangeListener;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.core.attribute.AppliedEnumValue;
import com.unitesk.requality.core.attribute.Attribute;
import com.unitesk.requality.core.attribute.Attribute.ParameterAvailability;
import com.unitesk.requality.core.attribute.AttributeGenerator;
import com.unitesk.requality.core.attribute.AttributeType;
import com.unitesk.requality.core.attribute.EnumDefinitionValue;
import com.unitesk.requality.core.attribute.EnumValue;
import com.unitesk.requality.core.attribute.ListValue;
import com.unitesk.requality.core.transaction.TransactionStorage;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.ui.dialogs.EnumRetypeDeleteDialog;
import com.unitesk.requality.eclipse.views.atypeeditors.BoolValueEditor;
import com.unitesk.requality.eclipse.views.atypeeditors.EnumDefinitionValueEditor;
import com.unitesk.requality.eclipse.views.atypeeditors.EnumDefinitionValueEditor.EnumChangeAction;
import com.unitesk.requality.eclipse.views.atypeeditors.EnumDefinitionValueEditor.EnumDefValueResult;
import com.unitesk.requality.eclipse.views.atypeeditors.FloatValueEditor;
import com.unitesk.requality.eclipse.views.atypeeditors.GeneratorValueEditor;
import com.unitesk.requality.eclipse.views.atypeeditors.IntValueEditor;
import com.unitesk.requality.eclipse.views.atypeeditors.ListValueEditor;
import com.unitesk.requality.eclipse.views.atypeeditors.ReferenceValueEditor;
import com.unitesk.requality.nodetypes.VirtualNode;
import com.unitesk.requality.tools.DeepFirstTreeWalker;

public class AttributesPanel implements INodeChangeListener {
	private static String enumchar = "\u24BA";
	public enum APanelWorkMode{GLOBAL,LOCAL};
	private boolean transactionIgnore;
	private String ignoreNameError=null;
	private static String MES_ENUM_DEF_DEL = "Altered ENUM definition is used in {cnt} ENUMs. Please select actions to be performed for all derived ENUMs:\n\t"
    					 + "'Remove' to remove all ENUMs\n\t'Change type' to change "
    					 + "from ENUM type to String\n\t'Cancel' to abort process",
    					 MES_ENUM_DEF_ID = "Altered ENUM definition is used in {cnt} ENUMs. Please select actions to be performed for all derived ENUMs:\n\t"						 
    					 + "'Rename' to change ENUM definition id to altered one\n\t'Remove' to remove all ENUMs\n\t'Change type' to change "
						 + "from ENUM type to String\n\t'Cancel' to abort process";
	private TreeNode getEnumDefStorage(){
		return VirtualNode.getRootTree(node.getTreeDB()).getRootNode();
	}
	class TableLabelProvider implements ITableLabelProvider,ITableColorProvider {
        @Override
        public void removeListener(ILabelProviderListener arg0) {
        }

        @Override
        public void addListener(ILabelProviderListener arg0) {
        }

        @Override
        public boolean isLabelProperty(Object arg0, String arg1) {
            return false;
        }

        @Override
        public void dispose() {
        }
        
        @Override
        public String getColumnText(Object element, int columnIndex) {
            Attribute attr = (Attribute) element;
            if(attr.getKey().equals(""))
            	return "";
            if (columnIndex == 1) {
            	return attr.getKey();
            }
            if (columnIndex == 2) {
            	if(attr.getRawType().equals(AttributeType.ENUM)){
            		AppliedEnumValue aev = attr.getGRawValue();
            		//Object[] val = (Object[]) attr.getRawValue();
            		if(aev!=null){
            			//String key = aev.getBaseKey();
            			//Attribute b = node.getTreeDB().getRootNode().getAttribute(key);
            			return aev.getBaseKey();
            		}
            		return AttributeType.ENUM.toString();
            	}
                return attr.getResultType().toString();
            }
            if (columnIndex == 3) {
                if (attr == null || attr.getValue()==null) return "null";
                Object val = attr.getValue();
                if(attr.getRawType()==AttributeType.ENUM){
                	AppliedEnumValue aev = attr.getGRawValue();
            		if(aev!=null){
            			return Attribute.valueToString(aev.getType(), aev.getValue());
            		}
                }
                if(attr.getResultType() == AttributeType.LIST||attr.getResultType() == AttributeType.ENUM_DEFINITION)
                	val = attr.getRawValue();
        		if(attr.getResultType() == AttributeType.REFERENCE)
        			return node.getLinkLabel(attr.getKey());
                return Attribute.valueToString(attr.getResultType(), val);
            }
            if(columnIndex == 4&&mode == APanelWorkMode.LOCAL)
            	return attr.getAvailability().toString();
            if(columnIndex == 5||columnIndex==4&&mode == APanelWorkMode.GLOBAL)
            	return attr.getGenerator()==null?"":attr.getGenerator().toString();
            if(columnIndex == 0 || columnIndex == 6|| columnIndex == 7 || columnIndex == 5&&mode == APanelWorkMode.GLOBAL)
            	return "";
            return "n/a";
        }

        @Override
        public Image getColumnImage(Object element, int columnIndex) {
        	Attribute a = ((Attribute)element);
        	if(!a.getKey().equals("")){
        		if(columnIndex == 0 && !a.getOrigin().equals(node.getUUId()))
        			return PlatformUI.getWorkbench().getSharedImages()
        					.getImage(ISharedImages.IMG_TOOL_BACK);
        		if(columnIndex == (mode==APanelWorkMode.GLOBAL?5:6) && enabled)
        			return PlatformUI.getWorkbench().getSharedImages()
        					.getImage(ISharedImages.IMG_ELCL_REMOVE);
        	}
        	
        	return null;
        }

		@Override
		public Color getForeground(Object element, int columnIndex) {
        	return null;
		}

		@Override
		public Color getBackground(Object element, int columnIndex) {
			Attribute a = ((Attribute)element);
        	if(!a.getKey().equals(""))
        		if((columnIndex == 2 || columnIndex==3) && a.getRawType().equals(AttributeType.ENUM)){
        			AppliedEnumValue aev = a.getGRawValue();
        			//Object[]val = (Object[])a.getRawValue();
        			if(aev!=null){
        				String name = aev.getBaseKey();
        				Object nowval = aev.getValue();
        				Attribute b = getEnumDefStorage().getAttribute(name);
        				if(b !=null && columnIndex == 2)
        					return null;
        				if(b!=null && b.getRawType().equals(AttributeType.ENUM_DEFINITION)){
        					EnumDefinitionValue edv = b.getGRawValue();
        					if(edv.containsValue(nowval))
        						return null;
        				}
        			}
        			return new Color(attribTable.getTable().getDisplay(), new RGB(255,160,0));
        		}
        	return null;
		}
    }
	private Set<String> names;
	private static IAttributeFilter deffilter = new IAttributeFilter(){

		@Override
		public boolean filter(Attribute a) {
			return !a.getKey().startsWith("_");
		}
	};
    private final class AttributeTableContentProvider implements
            IStructuredContentProvider {
        @Override
        public Object[] getElements(Object input) {
            if (input instanceof TreeNode) {
                TreeNode node = (TreeNode) input;
                ArrayList<Attribute> data = new ArrayList<Attribute>();
                for (String key : node.getAttributeKeys()) {
                	Attribute a = node.getAttribute(key);
                    if (filter.filter(a)) {
                        data.add(a);
                    }
                }
                if(mode ==  APanelWorkMode.LOCAL)
                	data.add(new Attribute(node,AttributeType.STRING,"",AttributeType.getDefaultValue(AttributeType.STRING)));
                else
                	data.add(new Attribute(node,AttributeType.ENUM_DEFINITION,"",AttributeType.getDefaultValue(AttributeType.ENUM_DEFINITION)));
                //cmce
                return data.toArray(new Object[0]);
            }
            return null;
        }

        @Override
        public void dispose() {
        }

        @Override
        public void inputChanged(Viewer arg0, Object arg1, Object arg2) {
        }
    }
    public void setAttributeFilter(IAttributeFilter f){
    	filter = f;
    }
   
    private IAttributeFilter filter = deffilter;
    private Composite panel;
    private APanelWorkMode mode = APanelWorkMode.LOCAL;
    private static final String columnNames[] = new String[] { "Name", "Type", "Value", "Scope"};
    private TreeNode node;
    private TableViewer attribTable;
    private ComboBoxCellEditor enumsel;
    private DefaultToolTip enumtip,tabletip;
    private ToolTip ttenum;
    private boolean enabled = true;
    private class EnumDefUsageRes{
    	public Map<String,Map<TreeNode,Set<String>>> map;
    	public long summ;
    	public EnumDefUsageRes(Map<String,Map<TreeNode,Set<String>>>m,long s){
    		this.map = m;
    		this.summ = s;
    	}
    }
    private EnumDefUsageRes getEnumDefUsages(String name){
    	Map<String,Map<TreeNode,Set<String>>> mp = new HashMap<String,Map<TreeNode,Set<String>>>();
    	int cnt = 0;
		DeepFirstTreeWalker walker = new DeepFirstTreeWalker(node.getTreeDB().getRootNode());
		Iterator<TreeNode> it = walker.iterator();
		while(it.hasNext()){
			TreeNode node2 = it.next();
			for(Attribute b: node2.getAttributes().values())
				if(b.getRawType().equals(AttributeType.ENUM)){
					AppliedEnumValue aev = b.getGRawValue();
					if(aev!=null && aev.getBaseKey().equals(name)){
						cnt++;	
						String v = aev.getValue().toString();
						if(!mp.containsKey(v))
							mp.put(v, new HashMap<TreeNode,Set<String>>());
						if(!mp.get(v).containsKey(node2))
							mp.get(v).put(node2, new TreeSet<String>());
						mp.get(v).get(node2).add(b.getKey());
					}
				}
		}
		//null key to overall summ
		return new EnumDefUsageRes(mp,cnt);
    }
    private boolean applyEnumDefDialogResult(String enumDefKey, int res){
    	return applyEnumDefDialogResult(enumDefKey, res, enumDefKey);
    }
    private boolean applyEnumDefDialogResult(String enumDefKey, int res, String newId){
    	if(res == IDialogConstants.CANCEL_ID)
    		return false;
    	DeepFirstTreeWalker walker = new DeepFirstTreeWalker(node.getTreeDB().getRootNode());
		Iterator<TreeNode> it = walker.iterator();
		while(it.hasNext()){
			TreeNode node2 = it.next();
			boolean change = false;
			for(Attribute b: node2.getAttributes().values()){
				if(b.getRawType().equals(AttributeType.ENUM)&&((AppliedEnumValue)b.getRawValue()).getBaseKey().equals(enumDefKey))
				{
					change = true;
					if(res == EnumRetypeDeleteDialog.REMOVE)
						node2.removeAttribute(b.getKey());
					else if(res == EnumRetypeDeleteDialog.RETYPE){
						Attribute newa = new Attribute(node2, ((AppliedEnumValue)b.getRawValue()).getType(),b.getKey(),((AppliedEnumValue)b.getRawValue()).getValue());
						newa.setAvailability(b.getAvailability());
						node2.putAttribute(newa);
					}
					else if(res == EnumRetypeDeleteDialog.CHANGE_ID){
						AppliedEnumValue aev = b.getGRawValue();
						Attribute newa = new Attribute(b.getOrigin(),b.getRawType(),b.getKey(),
								new AppliedEnumValue(aev.getValue(), aev.getComment(), aev.getType(), newId));
						newa.setAvailability(b.getAvailability());
						node2.putAttribute(newa);
					}
				}
			}
			if(change)
				node2.saveAttributes();
		}
    	return true;
    }
    private boolean removeNode(Shell shell, String name,boolean startTransaction){
    	if(startTransaction&&!transactionIgnore)
    		//TODO may be message
    		if(!node.getTreeDB().startTransaction("Update "+node.getType(),true))
    			return false;
    	Attribute a = node.getAttribute(name);
    	int res = Dialog.CANCEL;
		if(a.getRawType().equals(AttributeType.ENUM_DEFINITION)){
			long cnt = getEnumDefUsages(a.getKey()).summ;
			if(cnt>0){
				EnumRetypeDeleteDialog d = new EnumRetypeDeleteDialog(shell,MES_ENUM_DEF_DEL.replace("{cnt}", Long.toString(cnt)));
				res = d.open();
				if(!applyEnumDefDialogResult(a.getKey(), res))
					return false;
			}
		}
        Attribute old = node.removeAttribute(name);
        node.saveAttributes();
        refreshTable(old);
        attribTable.getControl().setFocus();
        panel.notifyListeners(SWT.Modify, new Event());
        if(startTransaction&&!transactionIgnore)
        	node.getTreeDB().commit();
        return true;
    }
    private String[] getAvailableTypes(){
    	int correction = mode == APanelWorkMode.LOCAL?1:0;
    	AttributeType[]ats = AttributeType.visibleValues();
    	String[] ret = new String[ats.length-correction];
    	for(int i = 0;i<ret.length;i++)
    		ret[i] = ats[i].toString();
    	return ret;
    }
    private void removeAttribute(){
    	TableItem[] items = attribTable.getTable().getSelection();
    	String[] names = new String[items.length];
    	for(int i = 0;i<items.length;i++)
    		names[i] = (((Attribute)items[i].getData())).getKey();
    		
    	removeAttribute(panel.getShell(),names, false);
    }
    private boolean removeAttribute(Shell shell, String[] rawkeys, boolean b){
    	TableItem[] items = attribTable.getTable().getItems();
    	List<TableItem> filtered = new ArrayList<TableItem>();
    	Set<String> keys = new HashSet<String>(Arrays.asList(rawkeys));
    	for(TableItem ti:items)
    		if(keys.contains((((Attribute)ti.getData())).getKey()))
    			filtered.add(ti);
    	items = filtered.toArray(new TableItem[0]);
    	int cnt = 0;
    	for (int i = 0; i < items.length; i++) {
            Object attrName = ((Attribute) items[i].getData()).getKey();
            if(!attrName.equals(""))
            	cnt++;
    	}
    	if(cnt == 0)
    		return false; 
    	//TODO message on fail
    	if (!transactionIgnore && node.getTreeDB().getStorage() instanceof TransactionStorage)
    		return false;
        String mes = items.length <= 1 ? "Are you sure you want to remove selected attribute?"
                : "Are you sure you want to remove selected attributes?";

        if ((items.length > 0)
                && (MessageDialog.openQuestion(panel.getShell(),
                        "Attribute Editor", mes) == true)) {
        	if(transactionIgnore || node.getTreeDB().startTransaction("Update "+node.getType(),true)){
        		boolean n = true;
        		for (int i = 0; i < items.length; i++) {
        			Object attrName = ((Attribute) items[i].getData()).getKey();
        			if(attrName.equals(""))
        				continue;
        			if(!removeNode(attribTable.getControl().getShell(), attrName.toString(),b)){
        				if(!transactionIgnore)
        					node.getTreeDB().rollback();
        				return false;
        			}
        		}
        		if(!transactionIgnore)
        			node.getTreeDB().commit();

        		return true;
        	}
        }
        return false;
    }
    private Listener removeAttrLink = new Listener() {
        @Override
        public void handleEvent(Event event) {
            if ((event.type == SWT.KeyDown && event.keyCode == SWT.DEL)
                    || (event.type == SWT.Selection)) {
            	removeAttribute();
            }
        }
    };

    private Map<String, Object> makeAttrMap() {
        Map<String, Object> innerAttr = new HashMap<String, Object>();
        for (String key : node.getAttributeKeys()) {
            innerAttr.put(key, node.getAttributeValue(key));
        }
        return innerAttr;
    }
    private Map<String,Integer> order = new HashMap<String,Integer>();
    private int compare(Attribute a, Attribute b){
    	if(!order.containsKey(a.getKey()))
    		order.put(a.getKey(), a.getKey().equals("")?Integer.MAX_VALUE:order.size());
    	if(!order.containsKey(b.getKey()))
    		order.put(b.getKey(), b.getKey().equals("")?Integer.MAX_VALUE:order.size());
    	return order.get(a.getKey()).compareTo(order.get(b.getKey()));
    }
    private Map<AttributeType, CellEditor> valueEditors;
	private GeneratorValueEditor generatorEditor;
	private ComboBoxCellEditor scopeEditor;
	private ComboBoxCellEditor typeCombo;
	private ControlDecoration nameDecorator;
	public AttributesPanel(Composite parent) {
		this(parent,APanelWorkMode.LOCAL);
	}

    private void setEnumSelItems(String[] vals){
    	enumsel.setItems(vals);
    }
    final CellEditor[] editors;
    private ControlDecoration createDecorator(Control c) {
        ControlDecoration controlDecoration = new ControlDecoration(c,
                SWT.LEFT | SWT.TOP);
        FieldDecoration fieldDecoration = FieldDecorationRegistry.getDefault()
                .getFieldDecoration(FieldDecorationRegistry.DEC_ERROR);
        controlDecoration.setImage(fieldDecoration.getImage());
        return controlDecoration;
    }

    public AttributesPanel(final Composite parent,APanelWorkMode md) {
        GridLayout layout = new GridLayout(1, false);
        layout.marginHeight = 0;
        layout.marginWidth = 0;
    	this.mode = md;
    	
    	panel = new Composite(parent, SWT.NONE);
    	panel.setLayoutData(new GridData(GridData.FILL_BOTH));
        panel.setLayout(layout);
        panel.setBackground(parent.getBackground());

        Label label = new Label(panel, SWT.NONE);
        label.setText( "Attributes:");
        label.setBackground(panel.getBackground());
        //label.setLayoutData(new GridData());
        
        attribTable = new TableViewer(panel, SWT.H_SCROLL | SWT.V_SCROLL
                | SWT.BORDER | SWT.RESIZE |SWT.FULL_SELECTION );
        tabletip = new DefaultToolTip(attribTable.getControl());
        attribTable.getControl().addMouseMoveListener(new MouseMoveListener() {
			private String lasttext="";
			private long lasttime = -1;
			@Override
			public void mouseMove(MouseEvent e) {
				ViewerCell vc = attribTable.getCell(new Point(e.x,e.y));
				if(vc!=null){
					Attribute a =(Attribute) (vc.getElement());
					if(a!=null){
						String text = null;
						int columnIndex = vc.getColumnIndex();
						if(!a.getKey().equals(""))
			        		if((columnIndex == 2 || columnIndex==3) && a.getRawType().equals(AttributeType.ENUM)){
			        			AppliedEnumValue aev = (AppliedEnumValue)a.getRawValue();
			        			if(aev!=null){
			        				String name = aev.getBaseKey();
			        				Object nowval = aev.getValue();
			        				Attribute b = getEnumDefStorage().getAttribute(name);
			        				if(b == null)
			        					text = "Enum definition with name '"+name+"' is not found!";
			        				if(b!=null && b.getRawType().equals(AttributeType.ENUM_DEFINITION)){
			        					EnumDefinitionValue edv = b.getGRawValue();
			        					if(edv==null || !edv.containsValue(nowval)){
			        						text = "Definition of '"+name+"' does not contain value '"+nowval+"'";
			        					}else
			        					{
			        						Object comm = aev.getComment();
			        						if(comm !=null && columnIndex == 3)
			        							text = comm.toString();
			        					}
			        				}
			        				else
			        					text = "Attribute with name '"+name+"' has incorrect type";
			        				
			        			}
			        		}
						tabletip.activate();
						if(text == null)
							tabletip.deactivate();
						else if(!lasttext.equals(text)){
							tabletip.setText(text);
							lasttext = text;
						}
					}
				}
			}
		});
        attribTable.getControl().addKeyListener(new KeyListener() {
			
			@Override
			public void keyReleased(KeyEvent e) {
				// TODO Auto-generated method stub
			}
			
			@Override
			public void keyPressed(KeyEvent e) {
				// TODO Auto-generated method stub
				//if(e.keyCode == SWT.INSERT)
					//insertAttribute();
				if(e.keyCode == SWT.DEL)
					removeAttribute();
				if(e.keyCode == SWT.F2)
					changeEditor(1);
				
				//open editor
				//change editor
			}
		});					
        attribTable.setSorter(new ViewerSorter() {
            private int getVal(String s) {
                int a = 0;
                for (int i = 0; i < s.length() && s.charAt(i) >= '0'
                        && s.charAt(i) <= '9'; i++)
                    a = a * 10 + s.charAt(i) - '0';
                return a;
            }

            public int compare(Viewer viewer, Object e1, Object e2) {
                if(e1 instanceof Attribute && e2 instanceof Attribute){
                	Attribute a1=(Attribute)e1,a2=(Attribute)e2;
                	return AttributesPanel.this.compare(a1,a2);
                	//return TreeNodeComparator.compareStrings(a1.getKey(), a2.getKey());
                }
                return super.compare(viewer, e1, e2);
            }
        });
        GridData gridData = new GridData(GridData.FILL_BOTH);
        attribTable.getControl().setLayoutData(gridData);
        attribTable.getControl().addFocusListener(new FocusListener() {
			
			@Override
			public void focusLost(FocusEvent e) {
				// TODO Auto-generated method stub
			}
			
			@Override
			public void focusGained(FocusEvent e) {
				ISelection s = attribTable.getSelection();
				if(s == null || s.isEmpty()){
					attribTable.getTable().select(0);
				}
			}
		});
        ColumnViewerToolTipSupport.enableFor(attribTable, ToolTip.RECREATE);
        
        // Create columns
        int i = 0;
     // Editors
        editors = new CellEditor[columnNames.length-(mode==APanelWorkMode.GLOBAL?1:0)];
        editors[0] = new TextCellEditor(attribTable.getTable(),SWT.FULL_SELECTION);
        editors[0].setValidator(new ICellEditorValidator() {
			@Override
			public String isValid(Object value) {
				String s = (String) value;
				if(s.equals(""))
					return null;
				if(node.getAttribute(s)!=null && !s.equals(ignoreNameError))
					return "Attribute with the same name already exists";
				if(s.startsWith("_"))
					return "Attribute name cannot be started with '_' character";
				if(!Attribute.checkNameIsValid(s, false))
					return "Name contains illegal characters or starts with _ or $";
				return null;
			}
		});
        nameDecorator = createDecorator(editors[0].getControl());
		nameDecorator.hide();
		
        editors[0].getControl().addKeyListener(new KeyListener() {
			@Override
			public void keyReleased(KeyEvent e) {
				Object o = e.widget;
				if(o!=null)
					o = ((Text)o).getText();
				if(o == null)
					return;
				String s = o.toString();
				String msg = editors[0].getValidator().isValid(s);
				nameDecorator.hide();
				if(msg!=null){
					nameDecorator.setDescriptionText(msg);
					nameDecorator.show();
				}
			}
			
			@Override
			public void keyPressed(KeyEvent e) {
			}
		});
        enumsel = new ComboBoxCellEditor(attribTable.getTable(),new String[0],SWT.READ_ONLY);
    //    enumsel.setActivationStyle(ComboBoxCellEditor.DROP_DOWN_ON_PROGRAMMATIC_ACTIVATION);
        enumsel.getControl().addMouseTrackListener(new MouseTrackListener() {
			
			@Override
			public void mouseHover(MouseEvent e) {
				// TODO Auto-generated method stub
				
			}
			@Override
			public void mouseExit(MouseEvent e) {
			}
	        long last = (new Date()).getTime();
	        int lastnum = -1;
			@Override
			public void mouseEnter(MouseEvent e) {
				int cnt = 0;
				for(final Shell s : enumsel.getControl().getShell().getShells()){
					if(s.getChildren().length == 1 && (s.getChildren()[0] instanceof org.eclipse.swt.widgets.List)){
						final org.eclipse.swt.widgets.List l = (org.eclipse.swt.widgets.List)s.getChildren()[0];
						if(enumsel.getItems().length==l.getItemCount()){
							enumtip = new DefaultToolTip(l,DefaultToolTip.RECREATE, false);
					        enumtip.activate();
							//Find list
							l.addMouseMoveListener(new MouseMoveListener(){

								@Override
								public void mouseMove(MouseEvent e) {
									//l.getBounds().height
									ScrollBar sb = l.getVerticalBar();
									int realmax = sb.getMaximum() - sb.getThumb()-sb.getMinimum();
									int sel = sb.getSelection()-sb.getMinimum();
									//System.out.println((sel*100/realmax));
									//System.out.println(l.getSize().y+" "+l.getVerticalBar().getSelection()+" "+l.getVerticalBar().getThumb()+" "+l.getVerticalBar().getSelection()+" "+l.getVerticalBar().getMinimum());
									int span = realmax == 0?0:sel*(l.getItemCount()*l.getItemHeight()-l.getSize().y)/realmax;
									int num = (e.y+span)/l.getItemHeight();
									if(num>=l.getItemCount())
										num = 0;
									if(num!=lastnum){
										enumtip.deactivate();
										String tip = "";
										Attribute a = (Attribute) (attribTable.getTable().getSelection()[0].getData());
										AppliedEnumValue aev = a.getGRawValue();
										//Object[]vl = (Object[]) a.getRawValue();
										if(aev!=null){
											Attribute base = getEnumDefStorage().getAttribute(aev.getBaseKey());
											if(base!=null && base.getRawType()==AttributeType.ENUM_DEFINITION){
												EnumDefinitionValue edf = base.getGRawValue();
												if(num<edf.getValues().size()){
													String s = edf.getValues().get(num).getComment();
													if(s!=null){
														enumtip.setText(s);
														enumtip.activate();
													}
												}
											}
										}
										lastnum = num;
									}
								}
								
							});
						}
					}
				}
				// TODO Auto-generated method stub
			}
		});
		final KeyListener tabListener =new KeyListener() {
			
			@Override
			public void keyReleased(KeyEvent e) {
			}
			
			@Override
			public void keyPressed(KeyEvent e) {
				if(e.keyCode==SWT.ARROW_RIGHT &&e.stateMask==SWT.CONTROL)
					changeEditor(1);
				if(e.keyCode==SWT.ARROW_LEFT&&e.stateMask==SWT.CONTROL)
					changeEditor(-1);
					//changeEditor();
					
			}
		}; 
		 attribTable.getColumnViewerEditor().addEditorActivationListener(new ColumnViewerEditorActivationListener() {
				@Override
				public void beforeEditorDeactivated(
						ColumnViewerEditorDeactivationEvent event) {
					for(CellEditor o:getEditors())
						if(o!=null){
							o.getControl().removeKeyListener(tabListener);
							if(o.getControl() instanceof Composite)
								for(Control c: ((Composite)o.getControl()).getChildren())
									c.removeKeyListener(tabListener);
						}
					ignoreNameError = null;
				}
				
				@Override
				public void beforeEditorActivated(ColumnViewerEditorActivationEvent event) {
					Object o = event.getSource();
					if(o instanceof ViewerCell){
						int ci = ((ViewerCell)o).getColumnIndex();
						if(ci == 1){
							ignoreNameError = ((ViewerCell)o).getText();
						}
					}
				}
				
				@Override
				public void afterEditorDeactivated(ColumnViewerEditorDeactivationEvent event) {
					if(((Attribute)((ViewerCell)event.getSource()).getElement()).getRawType()==AttributeType.ENUM_DEFINITION)
						updateTypeCombo(null);
				}
				
				@Override
				public void afterEditorActivated(ColumnViewerEditorActivationEvent event) {
					// TODO Auto-generated method stub
					for(CellEditor o:getEditors())
						if(o!=null && o.isActivated()){
							o.getControl().addKeyListener(tabListener);
							if(o.getControl() instanceof Composite)
								for(Control c: ((Composite)o.getControl()).getChildren())
									c.addKeyListener(tabListener);
							break;
						}
				}
			});
        String[]types = getAvailableTypes();
        typeCombo = new ComboBoxCellEditor(attribTable.getTable(), types ,SWT.READ_ONLY);
        //typeCombo.setActivationStyle(ComboBoxCellEditor.DROP_DOWN_ON_PROGRAMMATIC_ACTIVATION);
        typeCombo.setValidator(new ICellEditorValidator() {
			@Override
			public String isValid(Object value) {
				if(value != null){
					String s = value.toString();
					
				}
				return null;
			}
		});
        editors[1] = typeCombo;
        String[] scopevars = new String[ParameterAvailability.values().length];
        int pai = 0;
        for(ParameterAvailability pa:ParameterAvailability.values())
        	scopevars[pai++] = pa.toString();
        
        scopeEditor = new ComboBoxCellEditor(attribTable.getTable(), types,SWT.FULL_SELECTION);
        //scopeEditor.setActivationStyle(ComboBoxCellEditor.DROP_DOWN_ON_PROGRAMMATIC_ACTIVATION);
        scopeEditor.setItems(scopevars);
        if(mode ==APanelWorkMode.LOCAL)
        	editors[3] = scopeEditor;
        valueEditors = new HashMap<AttributeType, CellEditor>();
        valueEditors.put(AttributeType.STRING, new TextCellEditor(attribTable.getTable(),SWT.FULL_SELECTION));
        valueEditors.put(AttributeType.BOOL, new BoolValueEditor(attribTable.getTable(),SWT.FULL_SELECTION));
        valueEditors.put(AttributeType.INT, new IntValueEditor(attribTable.getTable(),SWT.FULL_SELECTION));
        valueEditors.put(AttributeType.FLOAT, new FloatValueEditor(attribTable.getTable(),SWT.FULL_SELECTION));
        //valueEditors.put(AttributeType.RESOURCE, new ResourceValueEditor(attribTable.getTable(),SWT.FULL_SELECTION));
        //editors[3] = new TextCellEditor(attribTable.getTable(),SWT.FULL_SELECTION);
        //editors[4] = new TextCellEditor(attribTable.getTable(),SWT.FULL_SELECTION);
        TableViewerColumn clink =  new TableViewerColumn(attribTable,SWT.FULL_SELECTION);
        clink.getColumn().setResizable(false);
        clink.getColumn().setWidth(25);
        clink.setEditingSupport(new EditingSupport(attribTable) {
			@Override
			protected void setValue(Object element, Object value) {
			}
			
			@Override
			protected Object getValue(Object element) {
				return "";
			}
			
			@Override
			protected CellEditor getCellEditor(Object element) {
				return new TextCellEditor();
			}
			
			@Override
			protected boolean canEdit(Object element) {
				if(!enabled)
					return false;
				if(((Attribute)element).getKey().equals(""))
					return false;
				UUID uuid = ((Attribute)element).getOrigin();
				if(uuid == null || uuid == node.getUUId())
					return false;
				TreeNode nde = node.getTreeDB().getNode(uuid);
				final Map<String,List<TreeNode>> map = new HashMap<String, List<TreeNode>>();
				map.put(nde.getType(), new ArrayList<TreeNode>());
				map.get(nde.getType()).add(nde);
				Job j = new WorkspaceJob("Test") {
					@Override
					public IStatus runInWorkspace(IProgressMonitor monitor)
							throws CoreException {
						Activator.getDefault().setActiveNodes(map);
						return Status.OK_STATUS;
					}
				};
				j.schedule();
				//attribTable.remove(element);
				//removeNode(((Attribute)element).getKey(),true);
				return false;
			}
		});
    	i = 0;
    	attribTable.addDoubleClickListener(new IDoubleClickListener() {
			
			@Override
			public void doubleClick(DoubleClickEvent event) {
				updateSorting();
				panel.update();
				}
		});
    	for (String colName : columnNames) {
            if(i==3 && mode == APanelWorkMode.GLOBAL){
            	i++;
            	continue;
            }
            TableViewerColumn column = new TableViewerColumn(attribTable,
                    SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION);

            column.getColumn().setText(colName);
            column.getColumn().setWidth(100);
            column.getColumn().setResizable(true);
            column.getColumn().setMoveable(true);
            final int iv = i;
 
            column.getColumn().addListener(SWT.Selection, new Listener() {
    			@Override
    			public void handleEvent(Event event) {
    				updateSorting(sortingMethod == iv ? sortingMethod + 10 : iv);
    				refreshTable(null);
    			}
            });
        	column.setEditingSupport(new EditingSupport(column.getViewer()) {
				
				@Override
				protected void setValue(final Object element, Object value) {
					nameDecorator.hide();
					if(getCellEditor(element).getValidator()!=null && getCellEditor(element).getValidator().isValid(value)!=null || value == null){
						//TODO message
						return;
					}
					Attribute a = (Attribute)element;
					if(a.getKey().equals("")&&(iv!=0||(value.toString().trim()).equals("")))
						return;
					boolean started = false;
					if(!transactionIgnore)
						started = node.getTreeDB().startTransaction("Update "+node.getType(),true);
					if(transactionIgnore || started){
						String key = a.getKey();
						Attribute old=null;
						AttributeType at = a.getRawType();
						if(a.getGenerator()!=null)
							at = a.getGenerator().getAttributeType();
						ParameterAvailability replpar = null;
						if(mode == APanelWorkMode.GLOBAL)
							replpar = a.getRawType().equals(AttributeType.ENUM_DEFINITION)?
									ParameterAvailability.LOCAL:
									ParameterAvailability.SUBTREE;
						boolean changed = false;
						if(iv==0){
							if(!a.getKey().equals(""))
								old=node.removeAttribute(a.getKey());
							String newkey=(value.toString()).trim();
							Attribute b = new Attribute(node.getUUId(), a.getRawType(), newkey,a.getValue(), a.getRawValue(),a.getGenerator());
							if(mode == APanelWorkMode.GLOBAL)
								b.setAvailability(replpar);
							node.putAttribute(b);
							key=newkey;
							changed = true;
						}
						if(iv==1 && (Integer)value > -1){
							String selType = typeCombo.getItems()[(Integer)value];
							if(!selType.equals(a.getRawType().toString())&&
									(a.getRawType()!=AttributeType.ENUM || !(a.getRawValue()instanceof AppliedEnumValue) || 
									!selType.equals(enumchar+((AppliedEnumValue)(a.getGRawValue())).getBaseKey()))){
								String k = a.getKey();
								int ind = (Integer)value;
								AttributeType newt=AttributeType.ENUM;
								if(ind<getAvailableTypes().length)
									newt = AttributeType.valueOf(getAvailableTypes()[(Integer)value]);
	
								if(mode == APanelWorkMode.GLOBAL)
									replpar = newt.equals(AttributeType.ENUM_DEFINITION)?
										ParameterAvailability.LOCAL:
										ParameterAvailability.SUBTREE;
								//if(newt.equals(AttributeType.ENUM)&&!k.startsWith("*")&&getIndex(k)==-1)
									//k = "*"+k;
								Object val = AttributeType.getDefaultValue(newt);
								String defName = typeCombo.getItems()[ind];
								if(defName.startsWith(enumchar))
									defName = defName.substring(enumchar.length());
								Attribute a2 = getEnumDefStorage().getAttribute(defName);
								EnumDefinitionValue bval = null;
								List<EnumValue> v = null;
								if(a2!=null){
									bval = a2.getGRawValue();
									v = bval.getValues();
								}
								//Map<Object,String> v = null;
								if(newt == AttributeType.ENUM){
									String comm = null;
									Object vl = "";
									if(v!=null && v.size()>0)
									{
										EnumValue ev = v.iterator().next();
										comm = ev.getComment();
										vl = ev.getValue();
									}
									val = new AppliedEnumValue(vl,comm, a2.getListType(),defName);
								}
								Attribute b = new Attribute(node.getUUId(), newt, k, val);
								if(replpar == null)
									replpar = a.getAvailability();
								b.setAvailability(replpar);
								old = node.putAttribute(b);
								changed = true;
							}
						}
						if(iv==2){
							changed = true;
							if(a.getRawType().equals(AttributeType.ENUM) && (value instanceof Integer)){
								AppliedEnumValue aev = a.getGRawValue();
								Integer sel = (Integer)value;
								if(sel == -1)
									changed = false;
								else{
									String ret = enumsel.getItems()[sel];
									if(aev!=null){
										Attribute b = getEnumDefStorage().getAttribute(aev.getBaseKey());
										if(b!=null && b.getRawType() == AttributeType.ENUM_DEFINITION)
										{
											EnumDefinitionValue edv = b.getGRawValue();
											if(edv!=null)
											{
												if(edv.getValues().isEmpty())
													changed = false;
												else{
													EnumValue ev =  edv.getValues().get(sel);
													value = new AppliedEnumValue(ev.getValue(), ev.getComment(), aev.getType(), aev.getBaseKey());
												}
											}else
												changed = false;
										}else
											changed = false;
									}else{
										changed = false;	
									}
								}
							}
							if(a.getRawType()==AttributeType.ENUM_DEFINITION && (value instanceof EnumDefValueResult)){
								EnumDefValueResult res = (EnumDefValueResult)value;
								EnumDefUsageRes useres = getEnumDefUsages(a.getKey());
								value = new EnumDefinitionValue(res.type,res.values);
								for(String s:useres.map.keySet()){
									if(res.renames.containsKey(s)){
										for(TreeNode node:useres.map.get(s).keySet())
											if(node == null || node.isDisposed())
												continue;
											else
												for(String nm:useres.map.get(s).get(node)){
													Attribute atr = node.getAttribute(nm);
													if(atr == null)//not from here
														continue;
													AppliedEnumValue aev = (AppliedEnumValue)atr.getRawValue();
													String comm = aev.getComment();
													for(EnumValue ev:res.values)
														if(ev.getValue().toString().equals(res.renames.get(s))){
															comm = ev.getComment();
															break;
														}
													aev.updateCommentValue(res.renames.get(s),comm);
													//Object[] rw = (Object[]) atr.getRawValue();
													//rw[2] = res.renaes.get(s);
													ParameterAvailability pa = atr.getAvailability();
													atr = new Attribute(atr.getOrigin(), atr.getRawType(), atr.getKey(), aev);
													atr.setAvailability(pa);
													node.putAttribute(atr);
													node.saveAttributes();
												}
									}
								}
								for(String s:res.remact.keySet()){
									EnumChangeAction act = res.remact.get(s);
									for(TreeNode node:useres.map.get(s).keySet())
										if(node == null || node.isDisposed())
											continue;
										else
											for(String nm:useres.map.get(s).get(node)){
												Attribute atr = node.removeAttribute(nm);
												if(atr == null)//not from here
													continue;
												AppliedEnumValue aev =  (AppliedEnumValue) atr.getRawValue();
												//Object[]val = (Object[]) atr.getRawValue();
												if(act == EnumChangeAction.RETYPE){
													Attribute a2 = new Attribute(atr.getOrigin(), aev.getType(), atr.getKey(), aev.getValue());
													a2.setAvailability(atr.getAvailability());
													node.putAttribute(a2);
												}
												node.saveAttributes();
											}
								}
							}
							if(at!=AttributeType.ENUM_DEFINITION && at!=AttributeType.ENUM && !Attribute.checkValueByType(at, value))
								changed = false;
							if(changed){
								Attribute b = new Attribute(node.getUUId(), at, a.getKey(), value);
								if(replpar == null)
									replpar = a.getAvailability();
								b.setAvailability(replpar);
								old=node.putAttribute(b);
							}
						}
						if(iv==3&&mode == APanelWorkMode.LOCAL)
							if((Integer)value != a.getAvailability().ordinal()){
								Attribute nat = new Attribute(node.getUUId(), a.getRawType(), a.getKey(),a.getValue(), a.getRawValue(), a.getGenerator());
								nat.setAvailability(ParameterAvailability.values()[(Integer)value]);
								old=node.putAttribute(nat);
								changed = true;
							}
						
						if(iv==4||iv==3&&mode == APanelWorkMode.GLOBAL)
							if(a.getRawType().equals(AttributeType.GENERATED)){
								Attribute b = new Attribute(node.getUUId(), at, a.getKey(), a.getValue(), a.getRawValue(),(AttributeGenerator)value);
								if(replpar == null)
									replpar = a.getAvailability();
								b.setAvailability(replpar);
								old=node.putAttribute(b);
								changed = true;
							}
						
						Attribute now = node.getAttribute(key);
						if(changed &&( old==null && now!=null || now == null && old!=null || now!=null && (!now.equals(old))||now!=null && now.isGenerated())){
							dirty = true;
							if(a!=null&&!a.getKey().equals(""))
							{
								order.put(now.getKey(), order.get(a.getKey()));
								//order.remove(a.getKey());
							}else
								order.put(now.getKey(), order.size());
							if(old!=null && !old.getKey().equals("")&&old.getRawType()==AttributeType.ENUM_DEFINITION&&iv!=2){
								long cnt = getEnumDefUsages(old.getKey()).summ;
								if(cnt!=0)
								{
									boolean changedType = now==null || now.getRawType()!=AttributeType.ENUM_DEFINITION;
									String msg = changedType?MES_ENUM_DEF_DEL:MES_ENUM_DEF_ID;
									int dres = new EnumRetypeDeleteDialog(attribTable.getTable().getShell(),
											msg.replace("{cnt}", Long.toString(cnt)), 
											!changedType).open();
									if(!applyEnumDefDialogResult(old.getKey(), 
											dres,
											key)){
										node.saveAttributes();
										if(started)
											node.getTreeDB().rollback();
										return;
									}
								}
							}
							if(!changed)
								if(old == null)
									node.removeAttribute(a.getKey());
								else
									node.putAttribute(old);
							node.saveAttributes();
							if(started)
								if(changed)
									node.getTreeDB().commit();
								else
									node.getTreeDB().rollback();
						}else
							if(started)
								node.getTreeDB().rollback();
					}
					refreshTable(null);
			        attribTable.getControl().setFocus();
			        panel.notifyListeners(SWT.Modify, new Event());

				}
				
				@Override
				protected Object getValue(Object element) {
					if(element == null)
						return "";
					Attribute a = (Attribute)element;
					if(a.getKey().equals(""))
						return "";
					if(iv==0){
						return a.getKey();
					}
					if(iv==1){
						String name = a.getRawType().name();
						if(a.getRawType() == AttributeType.ENUM){
							AppliedEnumValue aev = (AppliedEnumValue)a.getRawValue();
							if(aev!=null)
								name = aev.getBaseKey();
						}
						for(int i = 0;i<typeCombo.getItems().length;i++)
							if(typeCombo.getItems()[i].equals(enumchar+name)||typeCombo.getItems()[i].equals(name))
								return i;
						return a.getResultType().ordinal();
					}
					if(iv==2){
						if(a.getRawType().equals(AttributeType.ENUM)){
							String[]vals = enumsel.getItems();
							AppliedEnumValue aev = a.getGRawValue();
							Object val = aev.getValue();
							for(int i = 0;i<vals.length;i++){
								if(vals[i].equals(val.toString()))
									return i;
							}
							//String[]rvals = Arrays.copyOf(vals, vals.length+1);
							//rvals[rvals.length-1]="*"+val;
							setEnumSelItems(vals);
							return 0;
						}
						if(a.getResultType().equals(AttributeType.BOOL))
							return a.getRawValue().toString().toLowerCase();
						if(a.getResultType()==AttributeType.ENUM_DEFINITION)
							((EnumDefinitionValueEditor)(valueEditors.get(AttributeType.ENUM_DEFINITION)))
								.setUsages( getEnumDefUsages(a.getKey()).map );
						if(a.getResultType() == AttributeType.LIST ||
								a.getResultType() == AttributeType.ENUM_DEFINITION)
							return Attribute.valueToString(a.getResultType(), a.getRawValue());
						return a.getRawValue().toString();
					}
					if(iv==3&&mode == APanelWorkMode.LOCAL)
						return a.getAvailability().ordinal();
					return "";
				}
				
				@Override
				protected CellEditor getCellEditor(Object element) {
					Attribute a = (Attribute)element;
					if(iv == 1){
						if(a.getRawType() == AttributeType.ENUM_DEFINITION){
							updateTypeCombo(a.getKey());
						}
					}
					if(iv == 2){
						if(a.getRawType().equals(AttributeType.ENUM)){
							AppliedEnumValue aev = a.getGRawValue();
							if(aev!=null){
								Attribute b = getEnumDefStorage().getAttribute(aev.getBaseKey());
								if(b!=null && b.getRawType()==AttributeType.ENUM_DEFINITION){
									EnumDefinitionValue vals = b.getGRawValue();
									String[]vret = new String[vals.getValues().size()];
									int i = 0 ;
									boolean find = false;
									for(EnumValue ev: vals.getValues()){
										vret[i++] = ev.getValue().toString();
										if(ev.equals(aev.getValue()))
											find = true;
									}
									setEnumSelItems(vret);
									return enumsel;
								//	enumsel.setValue(getIndex(a.getKey()));
								}
								setEnumSelItems(new String[]{});//+setts[2].toString()});
								return enumsel;
							}
						}
						CellEditor ce = valueEditors.get(a.getResultType());
						return ce;
					}
					return editors[iv];
				}
				
				@Override
				protected boolean canEdit(Object element) {
					if(!enabled)
						return false;
					return !((Attribute)element).getKey().equals("")&&iv<=3&&iv>=0
							||iv==0;//||iv==2&&((Attribute)element).getRawType().equals(AttributeType.STRING);
				}
    		});
        	i++;
        }
        TableViewerColumn cgen =  new TableViewerColumn(attribTable,SWT.FULL_SELECTION);
        cgen.getColumn().setWidth(100);
        cgen.getColumn().setText("Generator");
        cgen.getColumn().addListener(SWT.Selection, new Listener() {
			@Override
			public void handleEvent(Event event) {
				updateSorting(sortingMethod == 4 ? sortingMethod + 10 : 4);
				refreshTable(null);
			}
        });
        cgen.setEditingSupport(new EditingSupport(attribTable) {
			@Override
			protected void setValue(Object element, Object value) {
				if(value == null||!getCellEditor(element).isDirty())
					return;
				if(transactionIgnore || node.getTreeDB().startTransaction("Update "+node.getType(),true)){
					Attribute a = (Attribute) element;
					TreeNode master = node.getTreeDB().getNode((((AttributeGenerator) value)).getMasterUUId());
					Object v = ((AttributeGenerator) value).getValue(master);
					ParameterAvailability pa = a.getAvailability();
					a = new Attribute(a.getOrigin(), AttributeType.GENERATED, a.getKey(), v,v ,((AttributeGenerator) value).copy());
					a.setAvailability(pa);
					node.putAttribute(a);
					node.saveAttributes();
					if(!transactionIgnore)
						node.getTreeDB().commit();
					refreshTable(null);
					attribTable.getControl().setFocus();
					panel.notifyListeners(SWT.Modify, new Event());
				}
			}
			
			@Override
			protected Object getValue(Object element) {
				Attribute a = (Attribute) element;
				AttributeGenerator ag = a.getGenerator();
				if(ag!=null &&  a.getRawValue() instanceof ListValue)
					ag.setRandValues((ListValue) a.getRawValue());
				return ag;
			}
			
			@Override
			protected CellEditor getCellEditor(Object element) {
				return generatorEditor;
			}
			
			@Override
			protected boolean canEdit(Object element) {
				if(!enabled)
					return false;
				if(((Attribute)element).getKey().equals(""))
					return false;
				if(((Attribute)element).getRawType() == AttributeType.ENUM_DEFINITION)
					return false;
				//attribTable.remove(element);
				//removeNode(((Attribute)element).getKey(),true);
				return true;
			}
		});
    	
        TableViewerColumn cdel =  new TableViewerColumn(attribTable,SWT.FULL_SELECTION);
    	cdel.getColumn().setResizable(false);
    	cdel.getColumn().setWidth(25);
    	cdel.setEditingSupport(new EditingSupport(attribTable) {
			@Override
			protected void setValue(Object element, Object value) {

			}
			
			@Override
			protected Object getValue(Object element) {
				return "";
			}
			
			@Override
			protected CellEditor getCellEditor(Object element) {
				return new TextCellEditor();
			}
			
			@Override
			protected boolean canEdit(Object element) {
				if(!enabled)
					return false;
				if(((Attribute)element).getKey().equals("")||!enabled)
					return false;
				
				//if(transactionIgnore || node.getTreeDB().startTransaction("Update "+node.getType(),true)){
					if(removeAttribute(attribTable.getTable().getShell(),new String[]{((Attribute)element).getKey()},false)){
					//	if(!transactionIgnore)
						//	node.getTreeDB().rollback();
					//}else{
						attribTable.remove(element);
						dirty = true;
					}
				//}
				return false;
			}
		});
    	
    	TableViewerColumn empty = new TableViewerColumn(attribTable, SWT.FULL_SELECTION);
    	empty.getColumn().setWidth(10);
    	
        attribTable.setColumnProperties(columnNames);
        attribTable.getTable().setHeaderVisible(true);
        attribTable.getTable().setLinesVisible(true);
        attribTable.setCellEditors(editors);
        
        attribTable.setLabelProvider(new TableLabelProvider());
        attribTable.setContentProvider(new AttributeTableContentProvider());
   /*     attribTable.addDoubleClickListener(new IDoubleClickListener() {
            @Override
            public void doubleClick(DoubleClickEvent event) {
                Attribute attribute = (Attribute) ((IStructuredSelection) event
                        .getSelection()).getFirstElement();
                StructuredSelection s = (StructuredSelection) event.getSelection();
                if(attribute.isGenerated())
                	return;
                String savedOldName = (String) attribute.getKey();
                EditAttrDialog addAttrDialog = new EditAttrDialog(getControl()
                        .getShell(), attribute.getType() , (String) attribute.getKey(),
                        attribute.getValue(), makeAttrMap());
                if (addAttrDialog.open() == EditAttrDialog.OK) {
                    node.getTreeDB().startTransaction("Update "+node.getType());
                    if (!savedOldName.equals(addAttrDialog.getKey())) {
                        node.removeAttribute(savedOldName);
                    }
                    node.putAttribute(new Attribute(node, attribute.getType(),
                            addAttrDialog.getKey(), addAttrDialog.getValue()));
                    node.saveAttributes();
                    node.getTreeDB().commit();
                    attribTable.refresh();
                    attribTable.getControl().setFocus();
                    panel.notifyListeners(SWT.Modify, new Event());
                    attribTable.refresh();
                }
            }
        });*/
    }
    private CellEditor getEditor(int ind,AttributeType at){
    	switch(ind){
    		case 0: case 1: return attribTable.getCellEditors()[ind];
    		case 2: return valueEditors.get(at);
    		case 3: return mode ==APanelWorkMode.GLOBAL?generatorEditor: scopeEditor; 
    		case 4: return generatorEditor; 
    	}
    	return null;
    }
    private int getEditorMax(){
    	return mode == APanelWorkMode.GLOBAL?4:5;
    }
    private int getEditorIndex(CellEditor tr){
    	for(int i=0;i<attribTable.getCellEditors().length;i++){
    		CellEditor ce = attribTable.getCellEditors()[i];
    		if(ce!=null && tr.equals(ce))
    			return i;
    	}
    	for(CellEditor ce:valueEditors.values())
    		if(ce!=null && ce.equals(tr))
    			return 2;
    	if(tr.equals(enumsel))
    		return 2;
    	if(tr.equals(scopeEditor))
    		return 3;
    	if(tr.equals(generatorEditor))
    		return 4;
    	return -1;
    }
    protected CellEditor[] getEditors() {
    	Set<CellEditor> editors = new HashSet<CellEditor>();
    	editors.addAll(Arrays.asList(attribTable.getCellEditors()));
    	for(CellEditor ce:valueEditors.values())
    		editors.add(ce);
    	editors.add(generatorEditor);
    	editors.add(enumsel);
    	editors.add(scopeEditor);
		return editors.toArray(new CellEditor[0]);
	}
	protected void changeEditor(int shift) {
    	final StructuredSelection ss = (StructuredSelection) attribTable.getSelection();
		if(ss!=null && !ss.isEmpty()){
			final Attribute a = (Attribute) ss.getFirstElement();
			int[] ai = getActiveEditor(); 
			int j =ai[1];
			final int oj=ai[1];
			j = (j+shift+getEditorMax())%getEditorMax();
			if(j == 3 && mode == APanelWorkMode.GLOBAL && a.getRawType() == AttributeType.ENUM_DEFINITION)
				j = (j+shift+getEditorMax())%getEditorMax();
			if(j<0)
				j = 0;
			final int jsel = j+1;
			final int sel = attribTable.getTable().getSelectionIndex();
				Display.getCurrent().syncExec(new Runnable() {
					@Override
					public void run() {
						attribTable.editElement(attribTable.getElementAt(sel), -1);
						//attribTable.refresh(attribTable.getElementAt(sel));
						attribTable.editElement(attribTable.getElementAt(sel), jsel);
						//attribTable.editElement(a, jsel);
						//attribTable.refresh(attribTable.getElementAt(sel));
						//attribTable.editElement(attribTable.getElementAt(sel), jsel);
						//eds[jsel].activate();
						//eds[jsel].setFocus();
						//refreshTable();
					}
				});
				

		}
	}
	protected int getIndex(String key) {
    	Iterator<String> it = names.iterator();
    	int i = 0;
    	while(it.hasNext()){
    		String s = it.next();
    		if(s.equals(key))
    			return i;
    		i++;
    	}
    	if(i>names.size())
    		return -1;
		return i;
	}
    public Control getControl() {
        return panel;
    }
    private int sortingMethod = 0;
    private void updateSorting(){
    	updateSorting(sortingMethod);
    }
    private Comparator<Attribute> attrSorter = new Comparator<Attribute>() {

		@Override
		public int compare(Attribute o1, Attribute o2) {
			int s = sortingMethod % 10;
			boolean s2 = sortingMethod / 10==1;
			switch (s) {
			case 0:
				return s2?o2.getKey().compareTo(o1.getKey()):o1.getKey().compareTo(o2.getKey());
			case 1:
				return s2?o2.getResultType().compareTo(o1.getResultType()):o1.getResultType().compareTo(o2.getResultType());
			case 2:
				return s2?o2.getValue().toString().compareTo(o1.getValue().toString()):o1.getValue().toString().compareTo(o2.getValue().toString());
			case 3:
				return s2?o2.getAvailability().compareTo(o1.getAvailability()):o1.getAvailability().compareTo(o2.getAvailability());
			case 4:{
				AttributeGenerator a1 = o1.getGenerator();
				AttributeGenerator a2 = o2.getGenerator();
				if(a1 == null)
					return a2==null
								?0
								:s2
									?-1
									:1;
				else
					return a2==null
							?s2
									?1
									:0
							:a1.toString().compareTo(a2.toString());
			}
			default:
				break;
			}
			return 0;
		}
	};
    private void updateSorting(int sorting){
    	if(this.sortingMethod!=sorting)
    		order.clear();
    	this.sortingMethod = sorting;
    	if(this.node!=null){
    		Attribute[]s = node.getAttributes().values().toArray(new Attribute[0]);
    		Arrays.sort(s,attrSorter);
    		for(Attribute a:s)
    			order.put(a.getKey(), order.size());
    	}
    }
    public void setNode(TreeNode node) {
    	if(this.node!=null)
    		this.node.removeGuiChangeListener(this);
    	boolean b = !node.equals(this.node); 
        this.node = node;
    	if(b){
    		dirty = false;
    		order.clear();
    		updateSorting();
    	}
        node.addGuiChangeListener(this);
        //TODO temp;
        valueEditors.put(AttributeType.REFERENCE, new ReferenceValueEditor(node, attribTable.getTable(),SWT.FULL_SELECTION));
        valueEditors.put(AttributeType.LIST, new ListValueEditor(node, attribTable.getTable(),SWT.FULL_SELECTION));
        valueEditors.put(AttributeType.ENUM_DEFINITION, new EnumDefinitionValueEditor(node, attribTable.getTable(),SWT.FULL_SELECTION));
        generatorEditor = new GeneratorValueEditor(node, attribTable.getTable(), SWT.FULL_SELECTION);
        //valueEditors.put(AttributeType.GENERATED, );
        attribTable.setInput(node);
        TreeNode root = getEnumDefStorage();
        updateTypeCombo(null);
    }
//    private void setEnabledRec(boolean b, Control c){
    	
    private void updateTypeCombo(String ignore) {
    	names = new HashSet<String>();
        for(Attribute a:getEnumDefStorage().getAttributes().values()){
        	if(a.getRawType().equals(AttributeType.ENUM_DEFINITION)&&ProjectSettingsMainSection.filter.filter(a)
        			&& Attribute.checkValueByType(AttributeType.ENUM_DEFINITION, a.getValue())&&
        			!((EnumDefinitionValue)a.getValue()).getValues().isEmpty())
        		names.add(a.getKey());
        }
        String[]types = getAvailableTypes();
        String[]ret = new String[types.length+names.size()];
        int i = 0;
        for(String at:types)
        	ret[i++] = at;

        for(String s : names)
        	if(ignore == null || !ignore.equals(s))
        		ret[i++] = enumchar+s;
        i = ret.length-1;
        while(ret[i]==null)
        	i--;
        typeCombo.setItems(Arrays.copyOf(ret,i+1));	
	}
	//    	if(c instanceof Composite)
  //  		for(Control cn:((Composite)c).getChildren())
    //			setEnabledRec(b, cn);;
  //  }
    public void refreshTable(Object object){
    	if(object!=null)
    		attribTable.refresh(object);
    	else
    		attribTable.refresh();
    }
	private int[] getActiveEditor() {
		final CellEditor[]eds = getEditors();
		for(int i = 0 ;i<eds.length;i++)
			if(eds[i]!=null && eds[i].isActivated()){
				return new int[]{i,getEditorIndex(eds[i])};
			}
		return new int[]{-1,-1};
	}
	public void setEnabled(boolean b) {
		//getControl().setEnabled(b);
		if(enabled!=b){
			enabled = b;
			refreshTable(null);
		}
	}
	@Override
	public void attributeChange(final TreeNode source, final String what) {
		new UIJob("refresh attribute panel") {
			@Override
			public IStatus runInUIThread(IProgressMonitor monitor) {
				if(attribTable.getControl().isDisposed())
					source.removeChangeListener(AttributesPanel.this);
				else{
					for(TableItem ti : attribTable.getTable().getItems()){
						if(((Attribute)ti.getData()).getKey().equals(what)){
							refreshTable(ti.getData());
							break;
						}
					}
				}
				return Status.OK_STATUS;
			}
		}.schedule();;
	}
	private boolean dirty = false;
	@Override
	public void deleted(final TreeNode source) {
		new UIJob("refresh attribute panel") {
			@Override
			public IStatus runInUIThread(IProgressMonitor monitor) {
				if(attribTable.getControl().isDisposed())
					source.removeChangeListener(AttributesPanel.this);
				return Status.OK_STATUS;
			}
		}.schedule();
	}
	@Override
	public void moved(final TreeNode source, TreeNode oldParent, TreeNode newParent) {
		new UIJob("refresh attribute panel") {
			@Override
			public IStatus runInUIThread(IProgressMonitor monitor) {
				if(attribTable.getControl().isDisposed())
					source.removeChangeListener(AttributesPanel.this);
				return Status.OK_STATUS;
			}
		}.schedule();
	}
	public void setTransactionIgnore(boolean ti) {
		transactionIgnore = ti;
	}
	public boolean isDirty() {
		return dirty;
	}
}
