package com.unitesk.requality.eclipse.views.properties;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.jface.fieldassist.ControlDecoration;
import org.eclipse.jface.fieldassist.FieldDecoration;
import org.eclipse.jface.fieldassist.FieldDecorationRegistry;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import com.unitesk.requality.core.CoreUtils;
import com.unitesk.requality.core.ITreeLogic.IdContainsWrongChars;
import com.unitesk.requality.core.ITreeLogic.IdEmpty;
import com.unitesk.requality.core.ITreeLogic.SameIdExistsInChildren;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.views.properties.AttributesPanel.APanelWorkMode;
import com.unitesk.requality.nodetypes.Requirement;

public class ElementViewPanel{
	public interface IProblemListener{
		void problemAppears(String problem);
		void problemResolved(String problem);
	}
	private List<IProblemListener> listeners = new ArrayList<IProblemListener>();
	public void addProblemListener(IProblemListener pl){
		listeners.add(pl);
	}
	
	private NameEditPanel name;
	private Text id;
	private ControlDecoration nodeIdDecorator;
	private ElementDescriptionPanel desc;
	private AttributesPanel ap;
	private TreeNode node;
	private boolean transactionIgnore = false;
	private static Map<String, String> defl = null;
	public static Map<String,String> getDefaultLabels(){
		if(defl == null){
			defl = new HashMap<String,String>();
			defl.put("idLabel", "Id:");
			defl.put("nameLabel","Name:");
			defl.put("descriptionLabel","Description:");
		}
		return Collections.unmodifiableMap(defl);
	}
	public boolean hasProblems(){
		return nodeIdDecorator.isVisible();
	}
	public ElementViewPanel(Composite parent, int columns){
		this(parent,columns,getDefaultLabels(),null);
	}
	public ElementViewPanel(Composite parent, int columns,TreeNode node){
		this(parent,columns,getDefaultLabels(),node);
	}
	public void setTransactionIgnore(boolean ti){
		transactionIgnore = ti;
		if(name!=null)
			name.setTransactionIgnore(ti);
		if(ap!=null)
			ap.setTransactionIgnore(ti);
	}
	public ElementViewPanel(Composite parent, int columns,Map<String,String> labels,TreeNode nde){
		new Label(parent,SWT.NONE).setText(labels.get("idLabel"));
		id = new Text(parent, SWT.NONE);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = columns<3?1:columns-1;
		id.setLayoutData(gd);
        nodeIdDecorator = createDecorator(id);
        nodeIdDecorator.hide();
        id.addKeyListener(new KeyListener() {
			
			@Override
			public void keyReleased(KeyEvent e) {
				try {
	                if (node == null) {
	                    return;
	                }

	                if (node.getTreeDB().getTreeLogic().isIdValidExc(node, id.getText().trim())) {
	                	unshowIdProblem();
	                } else {
	                    nodeIdDecorator.setDescriptionText("Element Id is not correct and Element Id should not be empty");
	                    showIdProblem();
	                }
	            } catch (SameIdExistsInChildren e1) {
	                nodeIdDecorator.setDescriptionText("Element with the same Id already exists");
	                showIdProblem();
	            } catch (IdEmpty e1) {
	            	showIdProblem();
	                nodeIdDecorator.setDescriptionText("Element Id is empty");
	            } catch (IdContainsWrongChars e1) {
	            	showIdProblem();
	                nodeIdDecorator.setDescriptionText("Element Id contains wrong character - " + e1.getWrongCharacter());
	            }
			}
			
			@Override
			public void keyPressed(KeyEvent e) {}
		});
        id.addFocusListener(new FocusAdapter() {
        	public void focusLost(FocusEvent e) {
        		unshowIdProblem();
                if (!id.getText().equals(id.getText().trim())) {
                    id.setText(id.getText().trim());
                }
                if (node == null) {
                    return;
                }
                if (node.getId().equals(id.getText())) {
                    return;
                }
                if (!node.getQualifiedId().equals(Requirement.getTypeRootQId())) {
                    String newId = id.getText().trim();
                    if (node.getTreeDB().getTreeLogic().isIdValid(node, newId)&& (transactionIgnore || node.getTreeDB().startTransaction("Change "+CoreUtils.splitWords(node.getType())+" Id",true))) {
                        node.setId(newId.trim());
                        if(!transactionIgnore)
                        	node.getTreeDB().commit();
                    } else {
                        id.setText(node.getId());
                    }
                }
            }
        });
		new Label(parent,SWT.NONE).setText(labels.get("nameLabel"));
		name = new NameEditPanel(parent,SWT.NONE);
		name.setTransactionIgnore(transactionIgnore);
		name.getControl().setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = columns<3?1:columns;
		name.getControl().setLayoutData(gd);

		new Label(parent,SWT.NONE).setText(labels.get("descriptionLabel"));
		desc = new ElementDescriptionPanel(parent,  SWT.MULTI | SWT.BORDER | SWT.H_SCROLL
                | SWT.V_SCROLL);
		gd = new GridData(600,80);
		gd.heightHint = 80;
		desc.getControl().setLayoutData(gd);
		desc.getControl().addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent e) {
	            //if new attribute text is "", then remove this attribute rather than set it to "",
	            //because otherwise we get modified .json-files with empty attribute added,
	            //when simply navigating nodes in Requality Explorer and Properties View,
	            //and this is affecting .svn
	            if(node==null) {
	                return;
	            }
	            String oldDescr = node.getDescription();
	            if (oldDescr == null) {
	                oldDescr = "";
	            }
	            String newDescr = "";
	            if(desc.getDescriptionControl().getDocument()==null)
	            	return;
	            try {
	                newDescr = desc.getDescriptionControl().getDocument().get(0,
	                		desc.getDescriptionControl().getDocument().getLength());
	                newDescr = CoreUtils.htmlEncapsulate(newDescr);
	            } catch (BadLocationException ex) {
	                ex.printStackTrace();
	            }
	            if (oldDescr.equals(newDescr)) {
	                return;
	            }
	            if(transactionIgnore || node.getTreeDB().startTransaction("Update "+node.getTypeLabel(),true)){
	            	node.setDescription(newDescr);
	            	node.saveAttributes();
	            	if(!transactionIgnore)
	            		node.getTreeDB().commit();
	            }
	            else
	            	desc.getDescriptionControl().setDocument(new org.eclipse.jface.text.Document(node
	                .getDescription()));
	//TODO else cancel
	        }
		});
		setTreeNode(nde);
		if(node==null || node.hasUserAttributes()){
			ap = new AttributesPanel(parent,APanelWorkMode.LOCAL);
			gd = new GridData(GridData.FILL_BOTH);
			gd.heightHint = 200;
			gd.horizontalSpan = columns<2?1:columns;
			ap.getControl().setLayoutData(gd);
		}
	}
	public boolean isDirty(){
		String newDescr = "";
		boolean descEqual = true;
		if(desc!=null){
				try {
					newDescr =desc.getDescriptionControl().getDocument().get(0,
					desc.getDescriptionControl().getDocument().getLength());
				} catch (BadLocationException e) {
					e.printStackTrace();
				}
				newDescr = CoreUtils.htmlEncapsulate(newDescr);
				if(oldDescr == null)
					oldDescr = "";
				descEqual = newDescr.equals(oldDescr);
		}
		if(oldName == null)
			oldName = "";
		return (ap!=null&&ap.isDirty()) || !descEqual || !name.getControl().getText().equals(oldName) || !id.getText().equals(oldId); 
	}
	protected void showIdProblem() {
		nodeIdDecorator.show();
		for(IProblemListener pl:listeners)
			pl.problemAppears("id_problem");
	}
	protected void unshowIdProblem() {
		nodeIdDecorator.hide();
		for(IProblemListener pl:listeners)
			pl.problemResolved("id_problem");
	}
	private ControlDecoration createDecorator(Text text) {
        ControlDecoration controlDecoration = new ControlDecoration(text,
                SWT.LEFT | SWT.TOP);
        FieldDecoration fieldDecoration = FieldDecorationRegistry.getDefault()
                .getFieldDecoration(FieldDecorationRegistry.DEC_ERROR);
        controlDecoration.setImage(fieldDecoration.getImage());
        return controlDecoration;
    }
	private String oldDescr, oldName, oldId;
	public void setTreeNode(TreeNode node){
		if(nodeIdDecorator!=null)
			unshowIdProblem();
		this.node = node;
		if(node!=null)
			id.setText(node.getId());
		name.setTreeNode(node);
		if(node!=null)
			desc.setDocument(new org.eclipse.jface.text.Document(node
	                .getDescription()));
		oldDescr = node.getDescription();
		oldName = node.getName();
		oldId= node.getId();
		if(ap!=null){
			ap.getControl().setVisible(true);
			if(node.hasUserAttributes())
				ap.setNode(node);
			else
				//TODO remove
				ap.getControl().setVisible(false);
		}
	}

}
