/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.views.properties;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;

import com.unitesk.requality.core.IAttributeFilter;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.core.attribute.AppliedEnumValue;
import com.unitesk.requality.core.attribute.Attribute;
import com.unitesk.requality.core.attribute.AttributeType;
import com.unitesk.requality.core.attribute.EnumDefinitionValue;
import com.unitesk.requality.core.attribute.EnumValue;
import com.unitesk.requality.eclipse.ui.dialogs.EnumDefExtractDialog;
import com.unitesk.requality.eclipse.views.properties.AttributesPanel.APanelWorkMode;
import com.unitesk.requality.nodetypes.ReportData;
import com.unitesk.requality.tools.DeepFirstTreeWalker;

public class ProjectSettingsMainSection extends TreeNodePropertiesTab {
	
    private AttributesPanel attrPanel;
    private Button enumDefFromAttr;
    private static String TEXT_ENUM_FROM_ATTR="Extract enum definition from attributes",
    						TEXT_LIST_NODE_TEMPL="List Elements templates";
    public static IAttributeFilter filter = new IAttributeFilter() {
		
		@Override
		public boolean filter(Attribute a) {
			return !a.getKey().startsWith("_")&&!a.getKey().startsWith("@");
		}
	};
    
    private TreeNode selectedRoot;
    private boolean isData(){
    	return node.getType().equals(ReportData.TYPE_NAME);
    }
    @Override
    protected void setTreeNode(TreeNode node) {
    	selectedRoot = node;
        attrPanel.setNode(node);
        attrPanel.setEnabled(!isData());
    }
    @Override
    public void createControls(final Composite parent,
            TabbedPropertySheetPage aTabbedPropertySheetPage) {
        super.createControls(parent, aTabbedPropertySheetPage);
        parent.setLayout(new GridLayout(2, false));
        parent.setLayoutData(new GridData(GridData.FILL_BOTH));
        
        Composite buttons = new Composite(parent,SWT.NONE);
        buttons.setLayout(new FillLayout());
        
        enumDefFromAttr = new Button(buttons, SWT.NONE);
        enumDefFromAttr.setText(TEXT_ENUM_FROM_ATTR);
        enumDefFromAttr.addListener(SWT.MouseDown, new Listener() {
			@Override
			public void handleEvent(Event event) {
				TreeDB db = selectedRoot.getTreeDB();
				DeepFirstTreeWalker walker = new DeepFirstTreeWalker(selectedRoot);
				Iterator<TreeNode> iter = walker.iterator();
				Set<String> names = new HashSet<String>();
				while(iter.hasNext()){
					TreeNode node = iter.next();
					for(Attribute a:node.getAttributes().values())
						if(EnumDefExtractDialog.filter.filter(a))
							names.add(a.getKey());
				}
				EnumDefExtractDialog d = new EnumDefExtractDialog(parent.getShell(), db, names);
				if(d.open()==EnumDefExtractDialog.OK){
					String[] res = d.getResult();
					String defName = res[1],attrName = res[0];
					if(db.startTransaction("Create enum definition '"+defName+"'")){
						EnumDefinitionValue edv = new EnumDefinitionValue(AttributeType.STRING, 
								new ArrayList<EnumValue>());
						walker = new DeepFirstTreeWalker(selectedRoot);
						iter = walker.iterator();
						Set<Object> vals = new HashSet<Object>();
						while(iter.hasNext()){
							TreeNode node = iter.next();
							Attribute a = node.getAttribute(attrName);
							if(a!=null&&filter.filter(a)&&
									a.getRawType()!=AttributeType.RESOURCE &&
									a.getRawType()!=AttributeType.ENUM &&
									a.getRawType()!=AttributeType.ENUM_DEFINITION
									)
								vals.add(a.getValue().toString());
						}
						for(Object o:vals)
							edv.addEnumValue(o,null);
						Attribute enumDef = new Attribute(selectedRoot,AttributeType.ENUM_DEFINITION,defName,
								edv);
								//new Object[]{AttributeType.STRING,vals});
						selectedRoot.putAttribute(enumDef);
						selectedRoot.saveAttributes();
						iter = walker.iterator();
						while(iter.hasNext()){
							TreeNode node = iter.next();
							Attribute a = node.getAttribute(attrName);
							if(a!=null&&EnumDefExtractDialog.filter.filter(a)){
								String val = a.getValue().toString();
								Attribute newa=new Attribute(node, AttributeType.ENUM, attrName, new AppliedEnumValue(val, null, edv.getType(), defName));
								newa.setAvailability(a.getAvailability());
								node.putAttribute(newa);
								node.saveAttributes();
							}
						}
						db.commit();
					}
				}
			}
		});
        
        TabbedPropertySheetWidgetFactory wf = aTabbedPropertySheetPage.getWidgetFactory();
        Composite panelmid = new Composite(parent, SWT.NONE);
        GridData gd = new GridData(GridData.FILL_HORIZONTAL);
        gd.horizontalSpan = 2;
        panelmid.setLayoutData(gd);
        GridLayout gl = new GridLayout(5, false);
        gl.horizontalSpacing = gl.verticalSpacing  = 0;
        panelmid.setLayout(gl);
        panelmid.setBackground(parent.getBackground());
      
        attrPanel = new AttributesPanel(parent, APanelWorkMode.GLOBAL);
        GridData data = new GridData(GridData.FILL_BOTH);
        data.horizontalSpan = 2;
        attrPanel.getControl().setLayoutData(data);
        attrPanel.getControl().setBackground(parent.getBackground());
        attrPanel.setAttributeFilter(filter);
    }
    
}
