/*
 * Copyright 2009-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.report;

import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.core.TreesTracker;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.nodetypes.VirtualNode;
import com.unitesk.requality.tests.CoverageFromFile;
import com.unitesk.requality.tests.CoverageStorage;
import com.unitesk.requality.tests.ICoverageStorageSource;
import com.unitesk.requality.tools.RequalityCLI;

import freemarker.template.Configuration;
import freemarker.template.DefaultObjectWrapper;

public class ReportGenerator {
    private static boolean useStat;

	public static void clearOldImages(File f, String saveTo){
    	if(!f.exists())
			return;
		String fc = RequalityCLI.getFileContent(f);
    	RequalityCLI.deleteOldImages(fc, null, null, saveTo);
	}
    private static void hideInstancers(TreeNode node, List<TreeNode> hidden) {
		if(node.getType().equals(VirtualNode.TYPE_NAME)&&!(((VirtualNode)node).isHidden())){
			((VirtualNode)node).setHiddenState(true);
			hidden.add(node);
			node.saveAttributes();
			node.getTreeDB().updateNodeChildren(node.getParent(true),true);
		}
		for(UUID node2:node.getRawChildrenUUIDs())
			if(!VirtualNode.getVirtualStorage(node.getTreeDB()).isHidden(node2))
				hideInstancers(node.getTreeDB().getNode(node2),hidden);
	}
    public static void generateReport(Requirement rootReq, Template templ, Map<TemplateFile,Writer> twriters,
            String saveTo, Map<String, Object> userData, ClassLoader baseCL)
            throws Exception {
    	List<TreeNode> hidden = new ArrayList<TreeNode>();
        hideInstancers(rootReq,hidden);

        Configuration cfg = new Configuration();
        cfg.setObjectWrapper(new DefaultObjectWrapper());
        cfg.setDirectoryForTemplateLoading(new File(templ.getReportPath()));
        for (TemplateFile t : templ.getTemplates()) {
        	Writer out = twriters.get(t);
            
            try {
                Map<String, Object> data = new HashMap<String, Object>(userData);
                if (t.getDataClass().length() > 0) {
                    IReportDataSource repObject = t.instanceDataObject(baseCL);
                    data.putAll(repObject.getData());
                    data.put("report", repObject);
                } else {
                    data.put("report", new ReportTools());
                }
                data.put("reportLocation", saveTo);
                data.put("rootReq", rootReq);
                data.put("root", rootReq.getTreeDB().getRootNode());
                data.put("utils", new ReportUtils(rootReq.getTreeDB(),rootReq));
                
                CoverageStorage cs;
                if(data.containsKey("coverageStorageType")){
                	String type = (String) data.get("coverageStorageType"), setts = (String) data.get("coverageStorageSettings");
                	ICoverageStorageSource ics = Activator.getCoverageSource(type);
                	if(ics != null){
                		cs = CoverageStorage.getStorage(rootReq.getTreeDB(), ics);
                		if(setts!=null)
                			cs.setSettings(setts.split(","));
                	}else
                		cs = CoverageStorage.getStorage(rootReq.getTreeDB(), new CoverageFromFile());
                }else{
                	cs = CoverageStorage.getStorage(rootReq.getTreeDB(), new CoverageFromFile());
                	if(data.containsKey("coverageFilePath"))
                		cs.setSettings(new String[]{(String)data.get("coverageFilePath")});
                }
                data.put("coverage", cs);
                              
                freemarker.template.Template ftl = cfg.getTemplate(
                        t.getTemplatePath(), "UTF-8");
//                Statistics st = null;
//                if(cs.getSettings()!=null){
//                	st = new Statistics(rootReq.getTreeDB().getRootNode().getResourcesPath());
//                }
                Progress st = new Progress(rootReq.getTreeDB().getRootNode().getResourcesPath(), saveTo);
                data.put("progress", st);
                
                TreeDB db = rootReq.getTreeDB();
                while(db.isVirtual()&&VirtualNode.getBaseTree(db)!=null){
                	db=VirtualNode.getBaseTree(db);               	
                }
                if(useStat){
                	StatisticManager sm = new StatisticManager(cs, TreesTracker.getTracker().getProject(db), rootReq.getTreeDB().getRootNode().getResourcesPath());
                	data.put("statman", sm);
                }else{
                	StatisticManagerCLI sm = new StatisticManagerCLI(rootReq, cs);
                	data.put("statman", sm);                	
                }
                data.put("userData", userData);
                ftl.process(data, out);
                //TODO fix for report treeNode
                if(cs.getSettings(rootReq)!=null)
                	cs.saveCoverageToFile(saveTo + File.separator
                            + t.getOutputFileName()+"_lastCov.xml");
            } catch (RuntimeException e) {
                throw e;
            } finally {
                out.close();
                for(TreeNode nd : hidden)
                	try{
                		((VirtualNode)nd).setHiddenState(false);
                		nd.saveAttributes();
                		rootReq.getTreeDB().updateNodeChildren(nd.getParent(true), true);
                	}catch(Exception e){
                		e.printStackTrace();
                	}
            }
        }
    }
    
    public static void generateReport(Requirement rootReq, Template templ,
            String saveTo, Map<String, Object> userData, ClassLoader baseCL)
            throws Exception {
    	Map<TemplateFile, Writer> map = new HashMap<TemplateFile, Writer>();
    	for (TemplateFile t : templ.getTemplates()) {
            
    		File outFile = new File(saveTo + File.separator
                + t.getOutputFileName());
    		clearOldImages(outFile, saveTo+ File.separator);
    		outFile.getParentFile().mkdirs();
    		map.put(t, new OutputStreamWriter(new FileOutputStream(outFile),
                    "UTF-8"));
    	}
    	generateReport(rootReq, templ, map, saveTo, userData,baseCL);
    }

    public static void generateReport(Requirement rootReq, Template templ,
            String saveTo, Map<String, Object> userData) throws Exception {
    	useStat = false;
        generateReport(rootReq, templ, saveTo, userData,
                ClassLoader.getSystemClassLoader());
    }

    public static void generateReport(Requirement rootReq, Template templ,Map<TemplateFile,Writer> twriters,
            String saveTo, Map<String, Object> userData) throws Exception {
    	useStat = true;
        generateReport(rootReq, templ, twriters, saveTo, userData,
                ClassLoader.getSystemClassLoader());
    }

}
